!******************************************************************************!
!> This module provides all possible 'pure variables' (= no prefixes) for a
!!  given kind of simulation.
!!
!! IMPORTANT NOTE: When you create a new variable, do not forget to add this
!! variable
!! name into mus_derVarPos_type and also in mus_build_derVarPos routine.
!!
!! \author Simon Zimny
!
module mus_variable_module

  ! include treelm modules
  use env_module,                   only: rk, globalMaxLevels, long_k, labelLen
  use tem_aux_module,               only: tem_abort
  use tem_var_system_module,        only: tem_var_system_type, tem_var_append
  use tem_stencil_module,           only: tem_stencilHeader_type
  use tem_derived_module,           only: evalSpacetime, evalDiff, evalRelDiff,&
    &                                     evalGrad, evalMag, evalMagGrad
  use tem_var_module,               only: tem_var_type
  use tem_meshInfo_module,          only: tem_initMeshInfoList
  use tem_variable_module,          only: tem_variable_type,                   &
    &                                     tem_varError_nonAppend
  use tem_logging_module,           only: logUnit

  ! include musubi modules
  use mus_scheme_header_module,     only: mus_scheme_header_type
  use mus_derQuan_module,           only: deriveDensity_FromState,             &
    &                                     derivePressure_FromState,            &
    &                                     deriveVelocity_FromState,            &
    &                                     deriveMomentum_FromState,            &
    &                                     deriveEquil_FromState,               &
    &                                     deriveNonEquil_FromState,            &
    &                                     deriveEquil_FromMacro,               &
    &                                     deriveEquil_ps_fromMacro,            &
    &                                     deriveKineEner_FromState,            &
    &                                     deriveShearStress_FromState,         &
    &                                     deriveShearMag_FromState,            &
    &                                     deriveWSS_FromState,                 &
    &                                     deriveTemp_FromState,                &
    &                                     deriveVelMag_FromState,              &
    &                                     deriveStrainRate_FromState,          &
    &                                     deriveStrainRatenNwtn_FromState,     &
    &                                     deriveShearStressnNwtn_FromState,    &
    &                                     deriveShearRate
  use mus_derQuanIncomp_module,     only: deriveVelocityIncomp_FromState,      &
    &                                     derivePressureIncomp_FromState,      &
    &                                     deriveEquilIncomp_FromState,         &
    &                                     deriveNonEquilIncomp_FromState,      &
    &                                     deriveEquilIncomp_FromMacro,         &
    &                                     deriveKineEnerIncomp_FromState,      &
    &                                     deriveStrainRateIncomp_FromState,    &
    &                                     deriveStrainRateIncompnNwtn_FromState
  use mus_derQuanMSGas_module,      only: deriveMolFracMSGas_FromState,        &
    &                                     deriveVelocityMSGas_FromState,       &
    &                                     deriveVelMagMSGas_FromState,         &
    &                                     deriveEquilMSGas_FromState,          &
    &                                     deriveEquilMSGas_FromMacro,          &
    &                                     deriveEquilMSGas_2d_FromState,       &
    &                                     deriveEquilMSGas_2d_FromMacro
  use mus_derQuanPhysics_module,    only: derivePressurePhy_FromState,         &
    &                                     deriveKinePressPhy_FromState,        &
    &                                     deriveDensityPhy_FromState,          &
    &                                     deriveNumDensityPhy_FromState,       &
    &                                     deriveMoleFluxPhy_FromState,         &
    &                                     deriveVelocityPhy_FromState,         &
    &                                     deriveMomentumPhy_FromState,         &
    &                                     deriveWSSPhy_FromState,              &
    &                                     deriveKineEnerPhy_FromState,         &
    &                                     deriveShearStressPhy_FromState,      &
    &                                     deriveShearMagPhy_FromState,         &
    &                                     deriveVelMagPhy_FromState,           &
    &                                     deriveTempPhy_FromState,             &
    &                                     deriveStrainRatePhy_FromState,       &
    &                                     deriveShearRatePhy
  use mus_derQuanMSLiquid_module,   only: deriveNumDensityMSLiquid_FromState,  &
    &                                     derivePressureMSLiquid_FromState,    &
    &                                     deriveKinePressMSLiquid_FromState,   &
    &                                     deriveMoleFracMSLiquid_FromState,    &
    &                                     deriveMoleFluxMSLiquid_FromState,    &
    &                                     deriveVelocityMSLiquid_FromState,    &
    &                                     deriveMomentumMSLiquid_FromState,    &
    &                                     deriveEquilVelMSLiquid_FromMacro,    &
    &                                     deriveEquilVelMSLiquid_FromState,    &
    &                                     deriveVelMagMSLiquid_FromState,      &
    &                                     deriveEquilMSLiquid_FromState,       &
    &                                     deriveEquilMSLiquid_FromMacro,       &
    &                                     deriveEquilMSLiquid_2d_FromState,    &
    &                                     deriveEquilMSLiquid_2d_FromMacro


  implicit none
  private

  public :: mus_derVarPos_type
  public :: mus_init_varSys
  public :: mus_build_derVarList
  public :: mus_kill_derVarList
  public :: mus_store_derVarPos
  public :: mus_init_variables

  !> This type stores the position of each variable in the global sys
  type mus_derVarPos_type
    integer :: density           = -1
    integer :: numDensity        = -1
    integer :: pressure          = -1
    integer :: kinePress         = -1
    integer :: velocity          = -1
    integer :: momentum          = -1
    integer :: equilvelFromState = -1
    integer :: equilvelFromMacro = -1
    integer :: velMag            = -1
    integer :: shearStress       = -1
    integer :: strainRate        = -1
    integer :: strainRatenNwtn   = -1
    integer :: shearMag          = -1
    integer :: wss               = -1
    integer :: temperature       = -1
    integer :: equilFromState    = -1
    integer :: equilFromMacro    = -1
    integer :: nonEquilFromState = -1
    integer :: molefrac          = -1
    integer :: moleflux          = -1
    integer :: kineticEnergy     = -1
    integer :: shearRate         = -1
  end type mus_derVarPos_type

  ! pure variables, their dependencies and regular function pointers without prefixes
  ! in case one of the following is a state variable direct access will be granted

  ! density
  type(tem_var_type), SAVE :: density_var
  ! number density (density/molecular_weight)
  type(tem_var_type), SAVE :: numDensity_var
  ! velocity
  type(tem_var_type), SAVE :: velocity_var
  ! momentum
  type(tem_var_type), SAVE :: momentum_var
  ! equilibrium velocity from macro
  type(tem_var_type), SAVE :: equilvelFromMacro_var
  ! equilibrium velocity from state
  type(tem_var_type), SAVE :: equilvelFromState_var
  ! pressure
  type(tem_var_type), SAVE :: pressure_var
  ! kinematic mixture pressure
  type(tem_var_type), SAVE :: KinePress_var
  ! equilibrium from state
  type(tem_var_type), SAVE :: equilFromState_var
  ! equilibrium from state
  type(tem_var_type), SAVE :: nonEquilFromState_var
  ! equilibrium from Macroscopic variable
  type(tem_var_type), SAVE :: equilFromMacro_var
  ! shearStress
  type(tem_var_type), SAVE :: shearStress_var
  ! shearStress for nonNewtonian fluid
  type(tem_var_type), SAVE :: shearStressnNwtn_var
  ! shearStress magnitude
  type(tem_var_type), SAVE :: shearMag_var
  ! wall shear stress
  type(tem_var_type), SAVE :: wss_var
  ! mole fraction
  type(tem_var_type), SAVE :: molefrac_var
  ! mole flux
  type(tem_var_type), SAVE :: moleflux_var
  ! velocity magnitude
  type(tem_var_type), SAVE :: velMag_var
  ! kinetic energy E = 1/2 rho sqrt( ux^2 + uy^2 +uz^2 )
  type(tem_var_type), SAVE :: kineticEnergy_var
  ! temperature
  type(tem_var_type), SAVE :: temperature_var
  ! strain rate
  type(tem_var_type), SAVE :: strainRate_var
  ! strain rate nonNewtonian
  type(tem_var_type), SAVE :: strainRatenNwtn_var
  ! shear rate
  type(tem_var_type), SAVE :: shearRate_var

  ! Generic Variables
  ! difference bewteen two dependent variables
  type(tem_var_type), SAVE :: diff_var
  ! relative difference
  type(tem_var_type), SAVE :: relDiff_var
  ! magnitude
  type(tem_var_type), SAVE :: mag_var
  ! gradient
  type(tem_var_type), SAVE :: grad_var
  ! magnitude of gradient
  type(tem_var_type), SAVE :: magGrad_var
  ! number of generic variables
  integer, parameter, public :: mus_nGenVars = 5

  ! variable physical units
  ! density
  type(tem_var_type), SAVE :: densityPhy_var
  ! number density (density/molecular_weight)
  type(tem_var_type), SAVE :: numDensityPhy_var
  ! pressure
  type(tem_var_type), SAVE :: pressurePhy_var
  ! kinematic pressure
  type(tem_var_type), SAVE :: KinePressPhy_var
  ! velocity
  type(tem_var_type), SAVE :: velocityPhy_var
  ! velocity magnitude
  type(tem_var_type), SAVE :: velMagPhy_var
  ! momentum
  type(tem_var_type), SAVE :: momentumPhy_var
  ! shearStress
  type(tem_var_type), SAVE :: shearStressPhy_var
  ! shear Magnitude
  type(tem_var_type), SAVE :: shearMagPhy_var
  ! wall shear stress
  type(tem_var_type), SAVE :: wssPhy_var
  ! kinetic energy E = 1/2 rho sqrt( ux^2 + uy^2 +uz^2 )
  type(tem_var_type), SAVE :: kineticEnergyPhy_var
  ! temperature
  type(tem_var_type), SAVE :: temperaturePhy_var
  ! mole flux
  type(tem_var_type), SAVE :: molefluxPhy_var
  ! strain rate
  type(tem_var_type), SAVE :: strainRatePhy_var
  ! strain rate nonNewtonian
  type(tem_var_type), SAVE :: strainRatenNwtnPhy_var
  ! shear rate
  type(tem_var_type), SAVE :: shearRatePhy_var

  contains

!******************************************************************************!
  !> This routine initialize all derive quantity variables with theit label,
  !! nComponents and dependencies.
  !!
  !! Only the function pointer for density is set here since it is
  !! generic for all different scheme kinds.
  !! All other function pointers are initialized based on their scheme kind
  !! in mus_build_dervarList
  !!
  subroutine mus_init_variables( )
    !---------------------------------------------------------------------------

    ! density
    density_var%variable%mapping        = 1
    density_var%variable%label          = 'density'
    density_var%variable%nComponents    = 1
    if( .not. allocated(density_var%variable%depVars) ) then
      allocate(density_var%variable%depVars(1))
    end if
    density_var%variable%depVars(1)     = 'pdf'
    density_var%do => deriveDensity_FromState

    ! density
    densityPhy_var%variable%mapping     = 1
    densityPhy_var%variable%label       = 'density_phy'
    densityPhy_var%variable%nComponents = 1
    if( .not. allocated(densityPhy_var%variable%depVars) ) then
      allocate(densityPhy_var%variable%depVars(1))
    end if
    densityPhy_var%variable%depVars(1)  = 'density'
    densityPhy_var%do => deriveDensityPhy_FromState

    ! number density = density/molecular_weight
    numDensity_var%variable%mapping     = 1
    numDensity_var%variable%label       = 'numDensity'
    numDensity_var%variable%nComponents = 1
    if( .not. allocated(numDensity_var%variable%depVars) ) then
      allocate(numDensity_var%variable%depVars(1))
    end if
    numDensity_var%variable%depVars(1)  = 'pdf'
    numDensity_var%do => deriveNumDensityMSLiquid_FromState

    ! numDensity
    numDensityPhy_var%variable%mapping      = 1
    numDensityPhy_var%variable%label        = 'numDensity_phy'
    numDensityPhy_var%variable%nComponents  = 1
    if( .not. allocated(numDensityPhy_var%variable%depVars) ) then
      allocate(numDensityPhy_var%variable%depVars(1))
    end if
    numDensityPhy_var%variable%depVars(1)   = 'numDensity'
    numDensityPhy_var%do => deriveNumDensityPhy_FromState

    ! Pressure
    pressure_var%variable%mapping         = 1
    pressure_var%variable%label           = 'pressure'
    pressure_var%variable%nComponents     = 1
    if( .not. allocated(pressure_var%variable%depVars) ) then
      allocate(pressure_var%variable%depVars(1))
    end if
    pressure_var%variable%depVars(1)      = 'pdf'

    pressurePhy_var%variable%mapping      = 1
    pressurePhy_var%variable%label        = 'pressure_phy'
    pressurePhy_var%variable%nComponents  = 1
    if( .not. allocated(pressurePhy_var%variable%depVars) ) then
      allocate(pressurePhy_var%variable%depVars(1))
    end if
    ! change to kinematic pressure for multi-species liquid mixture
    pressurePhy_var%variable%depVars(1)   = 'pressure'
    pressurePhy_var%do => derivePressurePhy_FromState

    ! kinematic Pressure
    ! globsys depvar is depends on pdf
    ! tracking requVar depvar is depends on initial molar fraction
    ! spatial function defined through add_variable table
    KinePress_var%variable%mapping        = 1
    KinePress_var%variable%label          = 'kinematicpressure'
    KinePress_var%variable%nComponents    = 1
    if( .not. allocated(KinePress_var%variable%depVars) ) then
      allocate(KinePress_var%variable%depVars(2))
    end if
    KinePress_var%variable%depVars(1)     = 'pressure'
    KinePress_var%variable%depVars(2)     = 'density'

    KinePressPhy_var%variable%mapping     = 1
    KinePressPhy_var%variable%label       = 'kinematicpressure_phy'
    KinePressPhy_var%variable%nComponents = 1
    if( .not. allocated(KinePressPhy_var%variable%depVars) ) then
      allocate(KinePressPhy_var%variable%depVars(1))
    end if
    KinePressPhy_var%variable%depVars(1)  = 'kinematicpressure'

    ! velocity
    velocity_var%variable%label           = 'velocity'
    velocity_var%variable%nComponents     = 3
    if( .not. allocated(velocity_var%variable%depVars) ) then
      allocate(velocity_var%variable%depVars(1))
    end if
    velocity_var%variable%depVars(1)      = 'pdf'

    ! equilibrium velocity for multispecies
    equilvelFromState_var%variable%label        = 'equilvelFromState'
    equilvelFromState_var%variable%nComponents  = 3
    if( .not. allocated(equilvelFromState_var%variable%depVars) ) then
      allocate(equilvelFromState_var%variable%depVars(1))
    end if
    equilvelFromState_var%variable%depVars(1)   = 'pdf'

    ! equilibrium velocity for multispecies
    equilvelFromMacro_var%variable%label        = 'equilvelFromMacro'
    equilvelFromMacro_var%variable%nComponents  = 3
    if( .not. allocated(equilvelFromMacro_var%variable%depVars) ) then
      allocate(equilvelFromMacro_var%variable%depVars(1))
    end if
    equilvelFromMacro_var%variable%depVars(1)   = 'pdf'
    !\todo KM: 
    !equilFromMacro_var%variable%depVars(1) = 'density'
    !equilFromMacro_var%variable%depVars(2) = 'velocity'

    velocityPhy_var%variable%label              = 'velocity_phy'
    velocityPhy_var%variable%nComponents        = 3
    if( .not. allocated(velocityPhy_var%variable%depVars) ) then
      allocate(velocityPhy_var%variable%depVars(1))
    end if
    velocityPhy_var%variable%depVars(1)         = 'velocity'
    velocityPhy_var%do => deriveVelocityPhy_FromState

    ! momentum = density*velocity
    momentum_var%variable%label                 = 'momentum'
    momentum_var%variable%nComponents           = 3
    if( .not. allocated(momentum_var%variable%depVars) ) then
      allocate(momentum_var%variable%depVars(1))
    end if
    momentum_var%variable%depVars(1)            = 'pdf'

    momentumPhy_var%variable%label              = 'momentum_phy'
    momentumPhy_var%variable%nComponents        = 3
    if( .not. allocated(momentumPhy_var%variable%depVars) ) then
      allocate(momentumPhy_var%variable%depVars(1))
    end if
    momentumPhy_var%variable%depVars(1)         = 'momentum'
    momentumPhy_var%do => deriveMomentumPhy_FromState

    ! Temperature
    temperature_var%variable%mapping            = 1
    temperature_var%variable%label              = 'temperature'
    temperature_var%variable%nComponents        = 1
    if( .not. allocated(temperature_var%variable%depVars) ) then
      allocate(temperature_var%variable%depVars(1))
    end if
    temperature_var%variable%depVars(1)         = 'pdf'
    temperature_var%do => deriveTemp_FromState

    temperaturePhy_var%variable%mapping         = 1
    temperaturePhy_var%variable%label           = 'temperature_phy'
    temperaturePhy_var%variable%nComponents     = 1
    if( .not. allocated(temperaturePhy_var%variable%depVars) ) then
      allocate(temperaturePhy_var%variable%depVars(1))
    end if
    temperaturePhy_var%variable%depVars(1)      = 'temperature'
    temperaturePhy_var%do => deriveTempPhy_FromState

    ! equilibrium
    equilFromState_var%variable%label           = 'equilFromState'
    if( .not. allocated(equilFromState_var%variable%depVars) ) then
      allocate(equilFromState_var%variable%depVars(1))
    end if
    equilFromState_var%variable%depVars(1)      = 'pdf'

    ! non-equilibrium from pdf
    nonEquilFromState_var%variable%label        = 'nonEquilFromState'
    if( .not. allocated(nonEquilFromState_var%variable%depVars) ) then
      allocate(nonEquilFromState_var%variable%depVars(2))
    end if
    nonEquilFromState_var%variable%depVars(1)   = 'pdf'
    nonEquilFromState_var%variable%depVars(2)   = 'equilFromState'

    equilFromMacro_var%variable%label           = 'equilFromMacro'
    if( .not. allocated(equilFromMacro_var%variable%depVars) ) then
      allocate(equilFromMacro_var%variable%depVars(1))
    end if
    equilFromMacro_var%variable%depVars(1)      = 'pdf'
    !equilFromMacro_var%variable%depVars(1)     = 'density'
    !equilFromMacro_var%variable%depVars(2)     = 'velocity'

    ! shear stress
    shearStress_var%variable%label              = 'shearstress'
    shearStress_var%variable%nComponents        = 6
    if( .not. allocated(shearStress_var%variable%depVars) ) then
      allocate(shearStress_var%variable%depVars(1))
    end if
    shearStress_var%variable%depVars(1)         = 'nonEquilFromState'

    ! shear stress for nonNewtonian fluid
    shearStressnNwtn_var%variable%label         = 'shearstress'
    shearStressnNwtn_var%variable%nComponents   = 6
    if( .not. allocated(shearStressnNwtn_var%variable%depVars) ) then
      allocate(shearStressnNwtn_var%variable%depVars(2))
    end if
    shearStressnNwtn_var%variable%depVars(1)    = 'nonEquilFromState'
    shearStressnNwtn_var%variable%depVars(2)    = 'omega'

    ! shear stress phy that depends on shearstress
    shearStressPhy_var%variable%label           = 'shearstress_phy'
    shearStressPhy_var%variable%nComponents     = 6
    if( .not. allocated(shearStressPhy_var%variable%depVars) ) then
      allocate(shearStressPhy_var%variable%depVars(1))
    end if
    shearStressPhy_var%variable%depVars(1)      = 'shearstress'
    shearStressPhy_var%do => deriveShearStressPhy_FromState

    ! shear magnitude
    shearMag_var%variable%label                 = 'shearMag'
    shearMag_var%variable%nComponents           = 1
    if(.not. allocated(shearMag_var%variable%depVars) ) &
      allocate(shearMag_var%variable%depVars(1))
    shearMag_var%variable%depVars(1)            = 'shearstress'

    shearMagPhy_var%variable%label              = 'shearMag_phy'
    shearMagPhy_var%variable%nComponents        = 1
    if(.not. allocated(shearMagPhy_var%variable%depVars) ) &
      allocate(shearMagPhy_var%variable%depVars(1))
    shearMagPhy_var%variable%depVars(1)         = 'shearMag'
    shearMagPhy_var%do => deriveShearMagPhy_FromState

    ! wall shear stress
    wss_var%variable%label                      = 'wss'
    wss_var%variable%nComponents                = 1
    if( .not. allocated(wss_var%variable%depVars) ) then
      allocate(wss_var%variable%depVars(1))
    end if
    wss_var%variable%depVars(1)                 = 'shearstress'

    wssPhy_var%variable%label                   = 'wss_phy'
    wssPhy_var%variable%nComponents             = 1
    if( .not. allocated(wssPhy_var%variable%depVars) ) then
      allocate(wssPhy_var%variable%depVars(1))
    end if
    wssPhy_var%variable%depVars(1)              = 'wss'
    wssPhy_var%do => deriveWSSPhy_FromState

    ! strain rate
    strainRate_var%variable%label               = 'strainRate'
    strainRate_var%variable%nComponents         = 6
    if( .not. allocated(strainRate_var%variable%depVars) ) then
      allocate(strainRate_var%variable%depVars(2))
    end if
    strainRate_var%variable%depVars(1)          = 'pdf' ! to calculate density
    strainRate_var%variable%depVars(2)          = 'nonEquilFromState'

    ! strain rate nonNewtonian, but with same label
    strainRatenNwtn_var%variable%label          = 'strainRate'
    strainRatenNwtn_var%variable%nComponents    = 6
    if( .not. allocated(strainRatenNwtn_var%variable%depVars) ) then
      allocate(strainRatenNwtn_var%variable%depVars(3))
    end if
    strainRatenNwtn_var%variable%depVars(1)     = 'pdf' ! to calculate density
    strainRatenNwtn_var%variable%depVars(2)     = 'omega'
    strainRatenNwtn_var%variable%depVars(3)     = 'nonEquilFromState'

    ! strain rate phy that depends on strain rate
    strainRatePhy_var%variable%label            = 'strainRate_phy'
    strainRatePhy_var%variable%nComponents      = 6
    if( .not. allocated(strainRatePhy_var%variable%depVars) ) then
      allocate(strainRatePhy_var%variable%depVars(1))
    end if
    strainRatePhy_var%variable%depVars(1)       = 'strainRate'
    strainRatePhy_var%do => deriveStrainRatePhy_FromState

    ! shear rate
    shearRate_var%variable%label                = 'shearRate'
    shearRate_var%variable%nComponents          = 1
    if( .not. allocated(shearRate_var%variable%depVars) ) then
      allocate(shearRate_var%variable%depVars(1))
    end if
    shearRate_var%variable%depVars(1)           = 'strainRate'

    ! shear rate phy that depends on shearRate
    shearRatePhy_var%variable%label             = 'shearRate_phy'
    shearRatePhy_var%variable%nComponents       = 1
    if( .not. allocated(shearRatePhy_var%variable%depVars) ) then
      allocate(shearRatePhy_var%variable%depVars(1))
    end if
    shearRatePhy_var%variable%depVars(1)        = 'shearRate'
    shearRatePhy_var%do => deriveshearRatePhy

    ! velocity magnitude
    velMag_var%variable%label                   = 'velMag'
    velMag_var%variable%nComponents             = 1
    if(.not. allocated(velMag_var%variable%depVars) ) &
      allocate(velMag_var%variable%depVars(1))
    velMag_var%variable%depVars(1)              = 'velocity'

    velMagPhy_var%variable%label                = 'velMag_phy'
    velMagPhy_var%variable%nComponents          = 1
    if(.not. allocated(velMagPhy_var%variable%depVars) ) &
      allocate(velMagPhy_var%variable%depVars(1))
    velMagPhy_var%variable%depVars(1)           = 'velMag'
    velMagPhy_var%do         => deriveVelMagPhy_FromState

    ! mole fraction
    molefrac_var%variable%label                 = 'molefrac'
    molefrac_var%variable%nComponents           = 1
    if( .not. allocated(molefrac_var%variable%depVars) ) then
      allocate(molefrac_var%variable%depVars(1))
    end if
    molefrac_var%variable%depVars(1)            = 'pdf'

    ! mole flux(numDen*velocity)
    moleflux_var%variable%label                 = 'moleflux'
    moleflux_var%variable%nComponents           = 3
    if( .not. allocated(moleflux_var%variable%depVars) ) then
      allocate(moleflux_var%variable%depVars(1))
    end if
    moleflux_var%variable%depVars(1)            = 'pdf'

    ! mole flux(numDen*velocity)
    molefluxPhy_var%variable%label              = 'moleflux_phy'
    molefluxPhy_var%variable%nComponents        = 3
    if( .not. allocated(molefluxPhy_var%variable%depVars) ) then
      allocate(molefluxPhy_var%variable%depVars(1))
    end if
    molefluxPhy_var%variable%depVars(1)         = 'moleflux'

    ! kinetic energy
    kineticEnergy_var%variable%label            = 'kinetic_energy'
    kineticEnergy_var%variable%nComponents      = 1
    if( .not. allocated(kineticEnergy_var%variable%depVars) ) then
      allocate(kineticEnergy_var%variable%depVars(1))
    end if 
    kineticEnergy_var%variable%depVars(1)       = 'pdf'

    ! kinetic energy in physical units
    kineticEnergyPhy_var%variable%label         = 'kinetic_energy_phy'
    kineticEnergyPhy_var%variable%nComponents   = 1
    if( .not. allocated(kineticEnergyPhy_var%variable%depVars) ) then
      allocate(kineticEnergyPhy_var%variable%depVars(1))
    end if
    kineticEnergyPhy_var%variable%depVars(1)    = 'kinetic_energy'
    kineticEnergyPhy_var%do  => deriveKineEnerPhy_FromState

    ! difference between two scalar variables
    diff_var%variable%label                     = 'difference'
    diff_var%variable%nComponents               = 1
    diff_var%do => evalDiff

    ! relative difference between two scalar variables
    relDiff_var%variable%label                  = 'relDiff'
    relDiff_var%variable%nComponents            = 1
    relDiff_var%do => evalRelDiff

    ! magnitude of the dependent variable
    mag_var%variable%label                      = 'magnitude'
    mag_var%variable%nComponents                = 1
    mag_var%do => evalMag

    ! gradient of the dependent variable
    grad_var%variable%label                     = 'gradient'
    grad_var%variable%nComponents               = 3
    grad_var%do => evalGrad

    ! magnitude of gradient of the dependent variable
    magGrad_var%variable%label                  = 'magGrad'
    magGrad_var%variable%nComponents            = 1
    magGrad_var%do => evalMagGrad

  end subroutine mus_init_variables
!******************************************************************************!


!******************************************************************************!
  !> This routine build the derived variables depends on the scheme kind
  !!
  subroutine mus_build_derVarList(  derVarList, schemeHeader, stencil )
    !---------------------------------------------------------------------------
    !> array of all possible derived variables for this scheme kind
    !! (without prefixes)
    type( tem_var_type ), allocatable, intent(inout) :: derVarList(:)
    !> identifier of the scheme
    type( mus_scheme_header_type ), intent(in)       :: schemeHeader
    type( tem_stencilHeader_type ), intent(in)       :: stencil
    !---------------------------------------------------------------------------

    !set nComponents of any variable which are dependent on scheme
    equilFromState_var%variable%nComponents = stencil%QQ
    equilFromMacro_var%variable%nComponents = stencil%QQ
    nonEquilFromState_var%variable%nComponents = stencil%QQ

    write(logUnit(1),*)'Building the derived variable list ...'
    ! build the right variable list depending on the scheme kind
    select case( trim(schemeHeader%kind) )
      case('lbm', 'lbm_nNwtn')
        velocity_var%do          => deriveVelocity_FromState
        momentum_var%do          => deriveMomentum_FromState
        velMag_var%do            => deriveVelMag_FromState
        equilFromState_var%do    => deriveEquil_FromState
        nonEquilFromState_var%do => deriveNonEquil_FromState
        equilFromMacro_var%do    => deriveEquil_fromMacro
        shearStress_var%do       => deriveShearStress_FromState
        shearStressnNwtn_var%do  => deriveShearStressnNwtn_FromState
        shearMag_var%do          => deriveShearMag_FromState
        wss_var%do               => deriveWSS_FromState
        kineticEnergy_var%do     => deriveKineEner_FromState
        pressure_var%do          => derivePressure_FromState
        strainRate_var%do        => deriveStrainRate_FromState
        strainRatenNwtn_var%do   => deriveStrainRatenNwtn_FromState
        shearRate_var%do         => deriveShearRate

        allocate( derVarList( 27 ))
        if ( trim(schemeHeader%kind) == 'lbm' ) then
          ! Newtonian fluid
          derVarList = (/ density_var, velocity_var, pressure_var,  & ! 3
            &             equilFromState_var, equilFromMacro_var,   & ! 5
            &             nonEquilFromState_var, velMag_var,        & ! 7
            &             kineticEnergy_var, temperature_var,       & ! 9
            ! quantities that require omega
            &             shearStress_var, strainRate_var,          & ! 11
            ! quantities that depend on shearStress, strainRate
            &             shearMag_var, shearRate_var,              & ! 13
            &             wss_var, momentum_var,                    & ! 15
            ! quantities in physical units
            &             densityPhy_var, pressurePhy_var,          & ! 17
            &             velocityPhy_var, velMagPhy_var,           & ! 19
            &             momentumPhy_var,                          & ! 20
            &             shearStressPhy_var, shearMagPhy_var,      & ! 22
            &             wssPhy_var, shearRatePhy_var,             & ! 24
            &             strainRatePhy_var,                        & ! 25
            &             kineticEnergyPhy_var, temperaturePhy_var /) ! 27
        else if ( trim(schemeHeader%kind) == 'lbm_nNwtn' ) then
          ! nonNewtonian fluid
          write(logUnit(10),*)' Building derived variable list for nonNewtonian'
          derVarList = (/ density_var, velocity_var, pressure_var,  & ! 3
            &             equilFromState_var, equilFromMacro_var,   & ! 5
            &             nonEquilFromState_var, velMag_var,        & ! 7
            &             kineticEnergy_var, temperature_var,       & ! 9
            ! quantities that require omega
            &             shearStressnNwtn_var, strainRatenNwtn_var,& ! 11
            ! quantities that depend on shearStress, strainRate
            &             shearMag_var, shearRate_var,              & ! 13
            &             wss_var, momentum_var,                    & ! 15
            ! quantities in physical units
            &             densityPhy_var, pressurePhy_var,          & ! 17
            &             velocityPhy_var, velMagPhy_var,           & ! 19
            &             momentumPhy_var,                          & ! 20
            &             shearStressPhy_var, shearMagPhy_var,      & ! 22
            &             wssPhy_var, shearRatePhy_var,             & ! 24
            &             strainRatePhy_var,                        & ! 25
            &             kineticEnergyPhy_var, temperaturePhy_var /) ! 27
        end if ! schemeHeader == 'lbm' or 'lbm_nNwtn'

      case('lbm_incomp', 'lbm_incomp_nNwtn')
        velocity_var%do          => deriveVelocityIncomp_FromState
        velMag_var%do            => deriveVelMag_FromState
        momentum_var%do          => deriveMomentum_FromState
        equilFromState_var%do    => deriveEquilIncomp_FromState
        nonEquilFromState_var%do => deriveNonEquilIncomp_FromState
        equilFromMacro_var%do    => deriveEquilIncomp_fromMacro
        shearStress_var%do       => deriveShearStress_FromState
        shearMag_var%do          => deriveShearMag_FromState
        wss_var%do               => deriveWSS_FromState
        kineticEnergy_var%do     => deriveKineEnerIncomp_FromState
        pressure_var%do          => derivePressureIncomp_FromState
        strainRate_var%do        => deriveStrainRateIncomp_FromState
        shearRate_var%do         => deriveShearRate
        shearStressnNwtn_var%do  => deriveShearStressnNwtn_FromState
        strainRatenNwtn_var%do   => deriveStrainRateIncompnNwtn_FromState

        ! create derVarList
        allocate( derVarList( 25 ))
        if ( trim(schemeHeader%kind) == 'lbm_incomp' ) then
          write(logUnit(10),*) ' Building derived variable list for '//&
            &                   'lbm incomp'
          derVarList = (/ density_var, velocity_var, pressure_var,             &
            &             equilFromState_var, nonEquilFromState_var,           &
            &             equilFromMacro_var, shearStress_var, wss_var,        &
            &             shearMag_var, velMag_var, kineticEnergy_var,         &
            &             strainRate_var, shearRate_var, momentum_var,         &
            &             densityPhy_var, pressurePhy_var, velocityPhy_var,    &
            &             velMagPhy_var, strainRatePhy_var, shearRatePhy_var,  &
            &             shearStressPhy_var, shearMagPhy_var, wssPhy_var,     &
            &             kineticEnergyPhy_var, momentumPhy_var /)
        else if ( trim(schemeHeader%kind) == 'lbm_incomp_nNwtn' ) then
          write(logUnit(10),*) ' Building derived variable list for '//&
            &                   'lbm incomp nonNewtonian'
          derVarList = (/ density_var, velocity_var, pressure_var,             &
            &             equilFromState_var, nonEquilFromState_var,           &
            &             equilFromMacro_var, shearStressnNwtn_var, wss_var,   &
            &             shearMag_var, velMag_var, kineticEnergy_var,         &
            &             strainRatenNwtn_var, shearRate_var, momentum_var,    &
            &             densityPhy_var, pressurePhy_var, velocityPhy_var,    &
            &             velMagPhy_var, strainRatePhy_var, shearRatePhy_var,  &
            &             shearStressPhy_var, shearMagPhy_var, wssPhy_var,     &
            &             kineticEnergyPhy_var, momentumPhy_var /)
        end if ! schemeHeader == 'lbm_incomp' or 'lbm_incomp_nNwtn'

      case('lbm_ps')
        equilFromMacro_var%do => deriveEquil_ps_fromMacro
        allocate( derVarList( 3 ))
        derVarList = (/ density_var, velocity_var, equilFromMacro_var /)

      case('multi-species_gas')
        !\todo KM: change function pointer for velocity_var and equilibrium var to
        ! multispecies functions. i.e derive_velocityMultispecies - solve linear system
        ! of equation and equlibrium also differs for multispeces

        velocity_var%do  => deriveVelocityMSGas_FromState
        velMag_var%do    => deriveVelMagMSGas_FromState
        molefrac_var%do  => deriveMolfracMSGas_FromState
        select case(trim(stencil%label))
          case('d3q19')
            equilFromState_var%do => deriveEquilMSGas_FromState
            equilFromMacro_var%do => deriveEquilMSGas_fromMacro
          case('d2q9')
            equilFromState_var%do => deriveEquilMSGas_2d_FromState
            equilFromMacro_var%do => deriveEquilMSGas_2d_fromMacro
          case default
            write(logUnit(1),*)'No equilibrium function defined for '//        &
              &                'selected layout '//trim(stencil%label)//       &
              &                ' for scheme kind '//trim(schemeHeader%kind)
            call tem_abort()
        end select
        ! create derVarList
        allocate( derVarList( 5 ))
        derVarList = (/ density_var, velocity_var,                               &
          &             equilFromState_var,                                      &
          &             equilFromMacro_var, molefrac_var /)
      case('multi-species_liquid')
        pressure_var%do          => derivePressureMSLiquid_FromState
        velocity_var%do          => deriveVelocityMSLiquid_FromState
        momentum_var%do          => deriveMomentumMSLiquid_FromState
        equilvelFromMacro_var%do => deriveEquilVelMSLiquid_FromMacro
        equilvelFromState_var%do => deriveEquilVelMSLiquid_FromState
        !velMag_var%do            => deriveVelMagMSLiquid_FromState
        velMag_var%do            => deriveVelMag_FromState
        molefrac_var%do          => deriveMoleFracMSLiquid_FromState
        moleflux_var%do          => deriveMoleFluxMSLiquid_FromState
        molefluxPhy_var%do       => deriveMoleFluxPhy_FromState
        KinePress_var%do         => deriveKinePressMSLiquid_FromState
        KinePressPhy_var%do      => deriveKinePressPhy_FromState
        select case(trim(stencil%label))
          case('d3q19')
            equilFromState_var%do => deriveEquilMSLiquid_FromState
            equilFromMacro_var%do => deriveEquilMSLiquid_fromMacro
          case('d2q9')
            equilFromState_var%do => deriveEquilMSLiquid_2d_FromState
            equilFromMacro_var%do => deriveEquilMSLiquid_2d_fromMacro
          case default
            write(logUnit(1),*)'No equilibrium function defined for '//        &
              &                'selected layout '//trim(stencil%label)//       &
              &                ' for scheme kind '//trim(schemeHeader%kind)
            call tem_abort()
        end select
        ! create derVarList
        allocate( derVarList( 21 ))
        ! NOTE: pressure in this model is to compute mixture kinematic pressure
        ! not for species pressure, if used for species pressure than
        ! it returns only partial pressure which is invalid for liquids
        ! \todo implement nonEquilibrimFromState
        derVarList = (/ density_var, pressure_var, velocity_var,               &
          &             equilVelFromMacro_var, equilVelFromState_var,          &
          &             velMag_var, momentum_var,                              &
          &             equilFromMacro_var, equilFromState_var,                &
          &             KinePress_var, numDensity_var, molefrac_var,           &
          &             moleflux_var, densityPhy_var, pressurePhy_var,         &
          &             velocityPhy_var, velMagPhy_var, momentumPhy_var,       &
          &             numDensityPhy_var, KinePressPhy_var, molefluxPhy_var /)

      case default
        write(logUnit(1),*) ' The selected scheme kind is unknown '//          &
          &                 trim(schemeHeader%kind)
        call tem_abort()
    end select

  end subroutine mus_build_derVarList
!******************************************************************************!


!******************************************************************************!
  !> This routine deallocates the derived variables list
  !!
  subroutine mus_kill_derVarList( derVarList )
    !---------------------------------------------------------------------------
    !> array of all possible derived variables for this scheme kind
    !! (without prefixes)
    type(tem_var_type), allocatable, intent(inout) :: derVarList(:)
    !---------------------------------------------------------------------------
    deallocate( derVarList )

  end subroutine mus_kill_derVarList
!******************************************************************************!


!******************************************************************************!
!> Store the position of each variable in the global system in the derVarPos
!! This function is also called in Harvester.
  subroutine mus_store_derVarPos( derVarPos, var_sys, label )
    !---------------------------------------------------------------------------
    type(mus_derVarPos_type), intent(inout) :: derVarPos(:)
    !> global variable system
    type(tem_var_system_type), intent(in) :: var_sys
    !> array of field label prefix. Size=nFields
    character(len=*), intent(in) :: label(:)
    !------------------------------------------------------------------------
    integer :: iField, iVar, nVars, nFields
    character(len=labelLen) :: label_loc
    !------------------------------------------------------------------------
    nFields = size( label )
    nVars = var_sys%nVars

    ! loop over fields + 1 for mixture
    if ( nFields > 1 ) nVars = var_sys%nVars + 1

    write(logUnit(1),*) 'storing derived quantities position '
    do iVar = var_sys%nVars + 1, var_sys%Variable%nVals
      do iField = 1, nFields

        if ( iField > var_Sys%nVars ) then
          label_loc = '' ! mixture has no label
        else
          label_loc = trim(label(iField))
        end if

        if ( trim(label_loc)//'density' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%density = iVar
        else if ( trim(label_loc)//'numDensity' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%numDensity = iVar
        else if ( trim(label_loc)//'pressure' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%pressure = iVar
        else if ( trim(label_loc)//'kinematicpressure' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%kinePress = iVar
        else if ( trim(label_loc)//'velocity' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%velocity = iVar
        else if ( trim(label_loc)//'momentum' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%momentum = iVar
        else if ( trim(label_loc)//'equilvelFromState' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%equilvelFromState = iVar
        else if ( trim(label_loc)//'equilvelFromMacro' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%equilvelFromMacro = iVar
        else if ( trim(label_loc)//'equilFromState' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%equilFromState = iVar
        else if ( trim(label_loc)//'nonEquilFromState' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%nonEquilFromState = iVar
        else if ( trim(label_loc)//'equilFromMacro' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%equilFromMacro = iVar
        else if ( trim(label_loc)//'shearstress' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%shearstress = iVar
        else if ( trim(label_loc)//'wss' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%wss = iVar
        else if ( trim(label_loc)//'velMag' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%velMag = iVar
        else if ( trim(label_loc)//'molefrac' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%molefrac = iVar
        else if ( trim(label_loc)//'moleflux' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%moleflux = iVar
        else if ( trim(label_loc)//'kinetic_energy' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%kineticEnergy = iVar
        else if ( trim(label_loc)//'strainRate' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%strainRate = iVar
        else if ( trim(label_loc)//'strainRatenNwtn' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%strainRatenNwtn = iVar
        else if ( trim(label_loc)//'shearRate' == &
             & trim(var_sys%Variable%val(iVar)%label) ) then
           derVarPos(iField)%shearRate = iVar
        endif
      enddo
    end do

  end subroutine mus_store_derVarPos
!******************************************************************************!


!******************************************************************************!
  !> Register all variables in derVarList to var_sys. \n
  !! append labels as prefix.
  !!
  subroutine mus_init_varSys( derVarList, var_sys, label, nFields, tVar )
    !---------------------------------------------------------------------------
    !> array of all possible derived variables for this scheme kind
    !! (without prefixes)
    type(tem_var_type), intent(inout) :: derVarList(:)
    !> global variable system
    type(tem_var_system_type), intent(inout) :: var_sys
    !> array of field label prefix. Size=nFields
    character(len=*), intent(in) :: label(:)
    !> number of fields used in this simulation
    integer, intent(in) :: nFields
    !> additional variable defined in the lua file
    type( tem_variable_type ), intent(in) :: tVar(:)
    !---------------------------------------------------------------------------
    ! local variables
    integer :: pos, iVar, iField
    character(len=labelLen) :: buffer
    type(tem_var_type), allocatable :: meshInfoList(:)
    !---------------------------------------------------------------------------

    write(logUnit(1),*) 'Initializing derived quantities '

    ! Run over all variables
    do iVar = 1, size( derVarList )
      ! Register all derived variables in the global varSys
      ! Run over all the field labels
      do iField = 1, nFields
        ! Use field as a prefix for all the variables defined herefore
        write(buffer,'(a)') trim(label(iField))
        ! ... and append them to the global system with the prefix of the field
        !     in the label as well as in the dependencies
        call tem_var_append(                                                   &
          &       sys         = var_sys,                                       &
          &       varName     = trim(buffer)//derVarList(iVar)%variable%label, &
          &       nComponents = derVarList(iVar)%variable%nComponents,         &
          &       pos         = pos,                                           &
          &       depVar      = trim(buffer)//derVarList(iVar)%variable%depVars)

        ! check if the variable has been added (only appended if requirements are
        ! fullfilled)
        if ( pos == 0 ) then
          call tem_varError_nonAppend( var_prefix = buffer,                    &
            &                          variable = derVarList(ivar)%variable,   & 
            &                          fUnit = logUnit(1) )
        ! check if the variable to append is already a state variable
        else if ( pos > var_sys%nVars ) then
          var_sys%derive( pos )%do => derVarList( iVar )%do
          var_sys%variable%val( pos )%mapping = pos
        else if ( pos <= var_sys%nVars ) then
          write(logUnit(10),*) ' Derived variable ', &
            & derVarList(iVar)%variable%label, ' is already a state variable.'
        end if
      enddo ! loop labels

      ! Add mixture derived variables outside the label loop only if nVars > 1
      if ( nFields > 1 ) then
        ! if depvar is pdf then use 'all' for mixture 
        ! else use depVar defined for respective variable
        if ( trim(derVarList(iVar)%variable%depVars(1)) == 'pdf') then
          call tem_var_append( sys = var_sys,                                  &
            &              varName = derVarList(iVar)%variable%label,          &
            &              nComponents = derVarList(iVar)%variable%nComponents,&
            &              pos = pos,                                          &
            &              depVar = ['all'] )
        else
          call tem_var_append( sys = var_sys,                                  &
            &              varName = derVarList(iVar)%variable%label,          &
            &              nComponents = derVarList(iVar)%variable%nComponents,&
            &              pos = pos,                                          &
            &              depVar = derVarList(iVar)%variable%depVars )
        endif
        ! check if the variable has been added (only appended if requirements are
        ! fullfilled)
        if ( pos == 0 ) then
          call tem_varError_nonAppend( variable = derVarList(ivar)%variable,   & 
            &                          fUnit = logUnit(1) )
        ! check if the variable to append is already a state variable
        else if ( pos > var_sys%nVars ) then
          var_sys%derive( pos )%do => derVarList( iVar )%do
          var_sys%variable%val( pos )%mapping = pos
        end if
      end if ! nFields > 1
    end do ! loop derVarList

    !! \todo: this functionality is already moved to tem_append_inp_vars
    !! in tem_derived_module.
    ! Append space time variables to variable system
    do iVar = 1, size(tVar)
      if ( trim(tVar(iVar)%st_fun(1)%fun_kind) /= 'none' ) then
        call tem_var_append( sys = var_sys, var = tVar( iVar ), pos = pos)
        ! check if the variable has been added.
        ! only appended if requirements are fullfilled.
        if ( pos == 0 )then
          call tem_varError_nonAppend( variable = tVar(ivar),                  &
            &                          fUnit = logUnit(1) )
        ! check if the variable to append is already a state variable
        else if ( pos > var_sys%nVars ) then
          var_sys%derive( pos )%do => evalSpacetime
          var_sys%variable%val( pos )%mapping = pos
        end if
      end if
    enddo

    ! add diff_var variable in the end of the variable system since it is
    ! generic and does not need to be defined for each fields
    ! and it is need for all scheme kinds
    call tem_var_append( sys         = var_sys,                        &
      &                  varName     = diff_var%variable%label,        &
      &                  nComponents = diff_var%variable%nComponents,  &
      &                  pos         = pos )
    ! check if append successfully
    if ( pos > var_sys%nVars ) then
      var_sys%derive( pos )%do => diff_var%do
    end if

    ! append relative difference to var_sys
    call tem_var_append( sys         = var_sys,                          &
      &                  varName     = relDiff_var%variable%label,       &
      &                  nComponents = relDiff_var%variable%nComponents, &
      &                  pos         = pos )
    ! check if append successfully
    if ( pos > var_sys%nVars ) then
      var_sys%derive( pos )%do => relDiff_var%do
    end if

    ! append magnitude to var_sys
    call tem_var_append( sys         = var_sys,                      &
      &                  varName     = mag_var%variable%label,       &
      &                  nComponents = mag_var%variable%nComponents, &
      &                  pos         = pos )
    ! check if append successfully
    if ( pos > var_sys%nVars ) then
      var_sys%derive( pos )%do => mag_var%do
    end if

    ! append gradient to var_sys
    call tem_var_append( sys         = var_sys,                       &
      &                  varName     = grad_var%variable%label,       &
      &                  nComponents = grad_var%variable%nComponents, &
      &                  pos         = pos )
    ! check if append successfully
    if ( pos > var_sys%nVars ) then
      var_sys%derive( pos )%do => grad_var%do
    end if

    ! append magnitude of gradient to var_sys
    call tem_var_append( sys         = var_sys,                          &
      &                  varName     = magGrad_var%variable%label,       &
      &                  nComponents = magGrad_var%variable%nComponents, &
      &                  pos         = pos )
    ! check if append successfully
    if ( pos > var_sys%nVars ) then
      var_sys%derive( pos )%do => magGrad_var%do
    end if

    ! add the variables for mesh information
    call tem_initMeshInfoList( meshInfoList )
    do iVar = 1, size(meshInfoList)
      call tem_var_append( sys = var_sys, var = meshInfoList( iVar )%variable, &
        &                  pos = pos )

      ! check if the variable has been added (only appended if requirements are
      ! fullfilled)
      if ( pos == 0 )then
        call tem_varError_nonAppend( variable = meshInfoList(ivar)%variable,   &
            &                          fUnit = logUnit(1) )
      ! check if the variable to append is already a state variable
      else if ( pos > var_sys%nVars )then
        var_sys%derive( pos )%do => meshInfoList( iVar )%do
        var_sys%variable%val( pos )%mapping = pos
      end if
    enddo

    write(logUnit(10),*) ' Info about var system: ',trim(var_sys%systemName)
    write(logUnit(10),*) '  nVars (state vars) in varSys: ', var_Sys%nVars
    write(logUnit(10),*) '  nDerivedVars in varSys: ', var_Sys%variable%nVals

  end subroutine mus_init_varSys
!******************************************************************************!


end module mus_variable_module
!******************************************************************************!
