! We include two accesses to the pdf variables
! - array of structures (AOS)
! - structure of arrays (SOA)
!
??IFNDEF (SOA) THEN
?? LOGICAL :: SOA = .false.
??ENDIF
!
! Streaming layout
!   PUSH:  read element-local pdf and store to adjacent element
!   PULL:  read adjacent element pdf and store to local element
! Multi-level only works with Pull
!
!
??IFNDEF (STREAM_PUSH) THEN
?? LOGICAL :: STREAM_PUSH = .false.
??ENDIF

! NgDir: The ELEMENT direction with which the streaming operation takes places
!        Mainly used in the reconstruct_connectivity routine to build the neighbor array.
!        Also used in boundary routine
?? if(STREAM_PUSH) then
?? macro :: FETCH(iDir,node, QQ)= ?IDX?(?iDir?,?node?,?QQ?)
?? macro :: SAVE(iDir,node,QQ,neigh_array)=?neigh_array?(  ?NGPOS?( ?iDir?, ?node?, ?QQ? ) )
?? macro :: StreamName = push
?? macro :: NgDir(iDir) = ?iDir?
?? macro :: InvNgDir(iDir, stencil) = ?stencil?%cxDirInv( ?iDir? )
?? macro :: SignSource = -
?? macro :: tPostColl = Next
?? macro :: tPreColl = Now 
?? else         !PULL
?? macro :: FETCH(iDir,node, QQ,neigh_array)=   ?neigh_array?( ?NGPOS?(?iDir?,?node?, ?QQ?) )
?? macro :: SAVE(iDir,node, QQ)= ?IDX?(?iDir?,?node?, ?QQ?)
?? macro :: StreamName = pull
?? macro :: NgDir(iDir, stencil) = ?stencil?%cxDirInv( ?iDir? )
?? macro :: InvNgDir(iDir) = ?iDir?
?? macro :: SignSource = +
?? macro :: tPostColl = Now
?? macro :: tPreColl = Next
?? endif
!
!
!
?? if(SOA) then
!--------------------------------------------
!    S O A - Structure of arrays layout
!-------------------------------------------
! These are the array index access macros
! Access to the state array
?? macro :: IDX(iDir,node, QQ)=( (?iDir?-1)*nElems_fluid + ?node? )
! Access to the connectivity array
?? macro :: NGPOS(iDir,node, QQ)=( (?node?-1)*?QQ? + ?iDir? )


?? else
!--------------------------------------------
!    A O S - Array of structures layout
!-------------------------------------------
! These are the array index access macros
! Access to the state array
?? macro :: IDX(iDir,node, QQ)=( (?node?-1)*?QQ? + ?iDir? )
! Access to the connectivity array
?? macro :: NGPOS(iDir,node, QQ)=( (?node?-1)*?QQ? + ?iDir? )
! Element position in treeIDlist
?? macro :: ElemIDX(index, QQ)=( int((?index?-1)/?QQ?)+ 1 )
! Direction
?? macro :: DirIDX(index, QQ)=( mod(?index?-1,?QQ?)+1 )
?? endif

!------------------------------------------------------------------------------
! LB Macros
!------------------------------------------------------------------------------
! Incompressible formulation of the equilibrium 
! You must provide 
! * an output array fEq( layout%stencil(1)%QQ )
! * an input quantity rho (density = sum(f_i))
! * an input quantity vel(1:3) (velocity = sum(c_i*f_i))
! * a stencil in the mus_scheme_layout_type
! * a reference density rho0 (usually =1 )
! It uses cs2inv = 3._rk from tem_param_module
! and the _rk real kind for double precision
?? text :: Eq_incomp(fEq, rho, vel, layout, rho0)
    do iDir = 1, ?layout?%stencil(1)%QQ
      ! calculate equilibrium density
      ?fEq?( iDir ) = ?layout?%weight( iDir )                                &
        &           *( ?rho? +  ?rho0?*( cs2inv*                             &
        & sum( real(?layout?%stencil(1)%cxDir(:,iDir),kind=rk)*?vel?(:))     &
        & + (sum( real(?layout?%stencil(1)%cxDir(:,iDir),kind=rk)*?vel?(:))  &
        & *  sum( real(?layout?%stencil(1)%cxDir(:,iDir),kind=rk)*?vel?(:))) &
        & *cs2inv*cs2inv*0.5d0                                               &
        & - sum( ?vel?(:)*?vel?(:))*0.5_rk*cs2inv ))
    enddo
?? end text Eq_incomp

! Compressible formulation of the equilibrium 
! You must provide 
! * an output array fEq( layout%stencil(1)%QQ )
! * an input quantity rho (density = sum(f_i))
! * an input quantity vel(1:3) (velocity = sum(c_i*f_i))
! * a stencil in the mus_scheme_layout_type
! It uses cs2inv = 3._rk from tem_param_module
! and the _rk real kind for double precision
?? text :: Eq_comp(fEq, rho, vel, layout)
    do iDir = 1, ?layout?%stencil(1)%QQ
      ! calculate equilibrium density
      ?fEq?( iDir ) = ?layout?%weight( iDir )                                &
        &           * ?rho?*( 1._rk +  ( cs2inv*                             &
        & sum( real(?layout?%stencil(1)%cxDir(:,iDir),kind=rk)*?vel?(:))     &
        & + (sum( real(?layout?%stencil(1)%cxDir(:,iDir),kind=rk)*?vel?(:))  &
        & *  sum( real(?layout?%stencil(1)%cxDir(:,iDir),kind=rk)*?vel?(:))) &
        & *cs2inv*cs2inv*0.5d0                                               &
        & - sum( ?vel?(:)*?vel?(:))*0.5_rk*cs2inv ))
    enddo
?? end text Eq_comp


!----------- Compute kernel header
?? text :: compute_routineHeader( computeName )
  !> This interface definition must match the compute kernel header interfaces
  !! defined in scheme/mus_scheme_type_module.f90
  subroutine ?computeName?(                                                    &
    &                                  iScheme, fieldProp, inState, outState,  &
    &                                  neigh, nStart, nEnd, level, layout,     &
    &                                  pdfGlobal, params, scheme, varSys, tree )
    !--------------------------------------------------------------------------- 
    !> current scheme
    integer, intent(in) :: iScheme
    !> fluid parameter
    type(mus_field_prop_type), intent(in) :: fieldProp(:)
    !> input  pdf vector
    real(kind=rk), dimension(:), intent(in)  :: inState
    !> output pdf vector
    real(kind=rk), dimension(:), intent(out) :: outState
    !> connectivity vector
    integer, dimension(:), intent(in) :: neigh
    !> start positions in elements list
    integer, intent(in) :: nStart
    !>   end positions in elements list
    integer, intent(in) :: nEnd
    !> current level
    integer,intent(in) :: level
    !> current layout
    type(mus_scheme_layout_type), intent(in) :: layout
    !> global info type
    type(pdf_globInfo_type), intent(in) :: pdfGlobal
    !> global parameters
    type(mus_param_type),intent(in) :: params
    !> array of all schemes
    type( mus_scheme_type ), intent(in) :: scheme(:)
    !> variable system definition
    type(tem_var_system_type), intent(in) :: varSys
    !> treelm mesh information
    type( treelmesh_type ), intent(in) :: tree
?? end text compute_routineHeader


!----------- Boundary header
?? text :: bc_routineHeader( bcName )
  !> This interface definition must match the boundary header interfaces
  !! defined in bc/mus_bc_header_module.f90
  subroutine ?bcName?( me, state, currT, lastT, bcBuffer, globBC, glob, tree,  &
    & iLevel, sim_time, neigh, layout, fieldProp, varPos, nScalars, varSys,    &
    & derVarPos, params, physics, iField, mixture, IC_fields, derVarPosAll )
    !---------------------------------------------------------------------------
    class(boundary_type) :: me     !< global boundary type
    !> Current state vector of iLevel 
    real(kind=rk),intent(inout) :: state(:,:) 
    !> current time step
    integer,intent(in) :: currT
    !> last time step
    integer,intent(in) :: lastT
    !> state values of boundary elements of all fields of iLevel
    real(kind=rk),intent(in) :: bcBuffer(:) 
    !>gloabl type contains iLevel descriptor
    type( pdf_global_type ), intent(in) :: glob
    type( treelmesh_type ), intent(in) ::tree !< global treelm mesh 
    !> fluid parameters and properties
    type( mus_field_prop_type ), intent(in) :: fieldProp
    !> stencil layout information
    type( mus_scheme_layout_type ),intent(in) ::layout 
    !> the level on which this boundary was invoked
    integer,intent(in) :: iLevel
    !> connectivity array corresponding to state vector
    integer,intent(in) :: neigh(:)       
    !> global time information
    type( tem_time_type ), intent(in)  :: sim_time  
    !> global parameters
    type(mus_param_type),intent(in) :: params   
    !> pointer to field variable in the state vector
    integer, intent(in) :: varPos(:)
    !> number of Scalars in the scheme var system
    integer, intent(in) :: nScalars
    !> scheme variable system
    type( tem_var_system_type ), intent(in) :: varSys
    !> position of derived quantities in varsys
    type( mus_derVarPos_type ), intent(in) :: derVarPos
    !> position of derived quantities in varsys
    type( mus_derVarPos_type ), intent(in) :: derVarPosAll(:)
    !> scheme global boundary type
    type( glob_boundary_type ), intent(in) :: globBC
    !> scheme global boundary type
    type( mus_physics_type ), intent(in) :: physics
    !> current field
    integer, intent(in) :: iField
    !> mixture info
    type(mus_mixture_type), intent(in) :: mixture
    !> InitialConditions of all fields
    type(tem_ini_condition_type), intent(in) :: IC_fields(:)
?? end text bc_routineHeader



!----------- Interpolation header
?? text :: interpolateRoutine_header( intpName )
  !> This interface definition must match the interpolation header interfaces
  !! defined in intp/mus_interpolate_header_module.f90
  subroutine ?intpName?(  pdf, glob, fieldProp, level, scheme,               &
    &                   layout, intp, ind, varSys, physics, derVarPos )
    !> pdf state arrays
    type( pdf_data_type ), intent(inout) :: pdf(:) 
    !> pdf state properties
    type( pdf_global_type ), intent(inout) :: glob 
    !> fluid and species parameters
    type( mus_field_prop_type ), intent(in) ::  fieldProp(:)
    !> the layout used
    type( mus_scheme_layout_type ), intent(in)   :: layout
    !> the current scheme 
    type( mus_scheme_type ), intent(in)   :: scheme
    !> interpolation method info
    type( interpolation_type ), intent(in)   :: intp
    !> indirection list
    type( grw_intArray_type ), intent(in)   :: ind
    !> my refinement level
    integer,intent(in) :: level
    !> global variable system
    type( tem_var_system_type ), intent(in) :: varSys
    !> physics type to convert lattice to physics SI unit and vice versa
    type(mus_physics_type), intent(in) :: physics 
    !> required variable system which maps to global varsys
    type( mus_derVarPos_type ), intent(in) :: derVarPos(:)
?? end text interpolateRoutine_header

