! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> Initialize LB related unit stuff.
!! todo: It seems this module is not used by anywhere.
module mus_physicsLB_module

  ! include treelm modules
  use env_module,         only: rk, globalMaxLevels
  use treelmesh_module,   only: treelmesh_type
  use tem_aux_module,     only: tem_abort
  use tem_logging_module, only: logUnit

  ! include musubi modules
  use mus_param_module, only: mus_param_type
  use mus_pdf_module,    only: pdf_globInfo_type

  implicit none

  private

  ! public :: mus_setLBdxdt

contains

!******************************************************************************!
  !> Initialize flow field 
  !!
  subroutine mus_setLBdxdt( pdfGlobal, params )
    !---------------------------------------------------------------------------
    type(mus_param_type), intent(in) :: params  !< Global parameters
    type(pdf_globInfo_type), intent(inout) :: pdfGlobal !< global info type
    !---------------------------------------------------------------------------
    integer :: iLevel
    real(kind=rk) :: dtLB(globalMaxLevels)
    !---------------------------------------------------------------------------
 
    ! Set LB dx
    do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
      pdfGlobal%dxLB( iLevel ) = 2._rk**( pdfGlobal%minLevel - iLevel )
    end do

    !--------------------------------------------------------------------------!
    ! \todo KM: This part is just used to cross check new way in computing
    ! dtLB from scaleFactor and must be removed.
    ! Set acoustic or diffusive scaling parameters
    select case( trim( params%scaling ))
    case('acoustic')
      ! Time step for each level: ratio 1:2 from level to level
      do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
        dtLB( iLevel ) = 2._rk**( pdfGlobal%minLevel - iLevel ) 
      end do
    case('diffusive')
      ! Time step for each level: ratio 1:4 from level to level
      do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
        dtLB( iLevel ) = 2._rk**(2*( pdfGlobal%minLevel - iLevel ))
      end do
    case default 
      write(logUnit(1),*)' Error: wrong scaling chosen!   '
      write(logUnit(1),*)' Choose either acoustic or diffusive   '
      call tem_abort()
    end select
    !--------------------------------------------------------------------------!

    do iLevel = pdfGlobal%minLevel, pdfGlobal%maxLevel
      pdfGlobal%dtLB( iLevel ) = real(params%scaleFactor, kind=rk)**( &
      & pdfGlobal%minLevel - iLevel ) 
      if( pdfGlobal%dtLB( iLevel ) /= dtLB( iLevel )) then
        write(logUnit(1),*)'error in dtLB computation. differs from old one.'
        call tem_abort()
      end if
    end do

  end subroutine mus_setLBdxdt
!******************************************************************************!


end module mus_physicsLB_module
!******************************************************************************!
