! See copyright notice in the COPYRIGHT file.
!******************************************************************************!
!> In this module we collect routines  and type definitions related to 
!! statistics for the runtime of the code. 
!!
!!
module mus_statistics_module
  ! include treelm modules
  use mpi
  use tem_comm_env_module, only: tem_comm_env_type
  use tem_logging_module,  only: logUnit

  implicit none
  private

  public :: mus_statistics_type
  public :: mus_calc_commAmount

  !> runtime statistics 
  !! this includes memory consumption, amount of data to communciate, 
  !! runtimes etc.
  type mus_statistics_type
    !> number of links to communicate
    !! Subset of nLinks_total
    integer :: nLinks_comm
    !> number of total links available which theoretically can be communciated
    integer :: nLinks_total
  end type mus_statistics_type

contains

!******************************************************************************!
  !> Calculate the number of links to be communicated
  !!
  subroutine mus_calc_commAmount( stat, proc )
    !---------------------------------------------------------------------------
    !> runtime statistic
    type( mus_statistics_type), intent(inout) :: stat
    !> mpi communication enviroment with mpi communicator
    type(tem_comm_env_type), intent(in) :: proc
    !--------------------------------------------------------------------------- 
    integer :: iErr
    integer :: nLinks_comm, nLinks_total
    !---------------------------------------------------------------------------

    if ( proc%comm_size > 1 ) then 
      call mpi_reduce( stat%nLinks_comm, nLinks_comm, 1, mpi_integer,          &
        &              mpi_sum, 0, proc%comm, iErr )
      call mpi_reduce( stat%nLinks_total, nLinks_total, 1,                     &
        &              mpi_integer, mpi_sum, 0, proc%comm, iErr )
      stat%nLinks_comm  = nLinks_comm
      stat%nLinks_total = nLinks_total

      if( nLinks_total > 0 ) then 
        write(logUnit(1),*) 'Communication amount compressed to [%]:',         &
          &          real( nLinks_comm )/real( nLinks_total )*100.
      endif
    endif
    
  end subroutine mus_calc_commAmount
!******************************************************************************!



end module mus_statistics_module
!******************************************************************************!
