-- Musubi configuration file. 
-- This is a LUA script.

-- Simulation name
simulation_name = 'simple_channel'
mesh = 'mesh/' -- Mesh information

-- Time step settigs
tmax           =  2500     -- total iteration number
interval       =  150
time = {useIterations = true,
        min = 1, max = tmax, interval = interval}


interpolation_method = 'linear'                     
umax = 0.095

scheme = { {

  identify = { label = 'fluid', kind = 'lbm', relaxation = 'bgk', layout = 'd3q19' },
  fluid = { omega = 1.891, rho0 = 1.0 },
  -- Initial condition 
  initial_condition = { density = 1.0, 
                      velocityX = 0.0,
                      velocityY = 0.0,
                      velocityZ = 0.0 } ,

-- Boundary conditions

  boundary_condition = {  
  { label = 'west', 
    kind = 'inlet_ubb', 
    velocityX = { kind = 'combined',
                transient= {predefined='smooth', min_factor = 0.0, max_factor=umax, from_time=0, to_time=5000}, 
                spatial = {predefined='parabol', 
                          shape = { kind = 'canoND', object = {center  = {-8.0,0.0,0.0},
                                                               halfvec = { {0.0,1.0,0.0}, {0.0,0.0,1.0}} } 
                                  }
                         }         
                                    
	      },
   velocityY = 0.0, velocityZ = 0.0, density = 1.0 },
{ label = 'east',
   kind = 'outlet_zero_prsgrd'}, 
{ label = 'north', 
   kind = 'wall' },
{ label = 'south', 
   kind = 'wall' },
{ label = 'top', 
   kind = 'wall' },
{ label = 'bottom', 
   kind = 'wall' },
{ label = 'obst', 
   kind = 'wall' }
 },  
-- Output settings
output = { active = true, -- VTK output activated?
  folder = 'output/',     -- Output location
  vtk = true,             -- VTK output activated?
  time = {min = 0, max = -1, interval = interval}
}, 
-- Tracking              
racking = {
  folder = 'tracking/', 
  label = 'probe_density', 
 variable = {{'density'}, {'wss'}}, 
 shape = {kind = 'canoND', object = {origin ={0.0,0.,0.} } },
 time = {min = 1, max = -1, interval = 1},
  format = 'ascii'      
 }

},

  -- second scheme
  {
  identify = { label = 'species', kind = 'lbm', relaxation = 'bgk', layout = 'd3q19' },
  depend = {variable = { {'density'}, {'velocity'} }, usescheme = 'fluid' }, 
  fluid = { omega = 1.58, rho0 = 1.0 },
  
  -- Initial condition 
  initial_condition = { density = 1.0, 
                      velocityX = 0.0,
                      velocityY = 0.0,
                      velocityZ = 0.0 } ,

-- Boundary conditions

  boundary_condition = {  
  { label = 'west', 
    kind = 'inlet_ubb', 
    velocityX = { kind = 'combined',
                transient= {predefined='smooth', min_factor = 0.0, max_factor=umax, from_time=0, to_time=5000}, 
                spatial = {predefined='parabol', 
                          shape = { kind = 'canoND', object = {center  = {-8.0,0.0,0.0},
                                                               halfvec = { {0.0,1.0,0.0}, {0.0,0.0,1.0}} } 
                                  }
                         }         
                                    
	      },
   velocityY = 0.0, velocityZ = 0.0, density = 1.0 },
{ label = 'east',
   kind = 'outlet_zero_prsgrd'}, 
{ label = 'north', 
   kind = 'wall' },
{ label = 'south', 
   kind = 'wall' },
{ label = 'top', 
   kind = 'wall' },
{ label = 'bottom', 
   kind = 'wall' },
{ label = 'obst', 
   kind = 'wall' }
 }, 


  }, 
} --Scheme description ends

--Geometry Increase definition
eomIncr = { { solidify = true,
               depend = { { useScheme = 'fluid', variable = { {'velocity'} },
                            condition = { { threshold = 0.0,  operator = '<'}} },
                         { seScheme = 'fluid', variable = { {'wss'}, {'velocity'} },
                            condition = { { threshold = 0.01, operator = '<'},
                                          { threshold = 0.0,  operator = '>='}} },
                        },
               time = {useIterations = true, min = tmax/3, max = tmax/3, interval = 1} 
             },
             { fluidify = false,
               depend = { { useScheme = 'fluid', variable = { {'wss'}, {'velocity'} },
                            condition = { { threshold = 0.01, operator = '>'},
                                          { threshold = 0.0,  operator = '<='}} },
                          { useScheme = 'species', variable = {'density'},
                            condition = { threshold = 0.001, operator = '!=' } }
                        },
               time = {useIterations = true, min = tmax/3, max = tmax/3, interval = 1} 
             }
}

eomIncr = { { solidify = true,
               depend = { {useScheme = 'fluid', variable = { 'wss' },
                           condition = {{ threshold = 0.0, operator = '<'}
                                       }}
                        },
               time = {useIterations = true, min = tmax/3, max = tmax/3, interval = 1}
             } 
}

geomIncr = { { solidify = true,
               depend = { {useScheme = 'fluid', variable = { {'velocity'}, {'wss'} },
                           condition = { { threshold = 0.0, operator = '<'},
                                         { threshold = 0.0, operator = '<'}
                                       }
                          }
                        },
               time = {useIterations = true, min = tmax/3, max = tmax/3, interval = 1}
             } 
}


