----------------------- PLEASE READ THIS ---------------------------!!!

-- This input file is set up to run for regression check
-- Please make sure you DO NOT MODIFY AND PUSH it to the repository
                                                                          
--------------------------------------------------------------------!!!


-- Musubi configuration file. 
-- This is a LUA script.

-- Simulation name
simulation_name = 'mini-channel'
mesh = 'mesh/'  -- Mesh information
-- Time step settings
tmax =  5000  -- total iteration number
interval = tmax/10
sim_control = {
  time_control = { 
    max = {iter=tmax},
    interval = {iter=interval}
  }  
}

ebug = { 
  verbosity    = 100,
  debugMode    = true, 
  debugFiles   = true,
  dumpTreeIDs  = true,
  dumpAuxLists = true,
  debugMesh    = './debug/mesh_', 
  debugStates  = { 
  write = {
    folder    = './debug/',    -- the folder the restart files are written to
    time_control = { useIterations = true,interval = 1, min = 1, max = tmax }
    }
 }} 

-- Initial condition 
rho0 = 1.
cs2 = 1./3.
initial_condition = { pressure = rho0*cs2,
                      velocityX = 0.0,
                      velocityY = 0.0,
                      velocityZ = 0.0 }

-- Boundary conditions
boundary_condition = {  
{ label = 'inlet', 
  kind = 'inlet_ubb', 
  velocityX = 0.06, velocityY = 0.0, velocityZ = 0.0 }
,{ label = 'wall', kind = 'wall'} 
,{ label = 'outlet', 
  kind = 'outlet_expol', pressure = rho0*cs2
  }
}

fluid = { omega = 1.8, rho0 = 1.0 }
identify = {
    label = '3D',
    kind = 'lbm', 
    layout = 'd3q19', 
    relaxation='bgk'
}

-- Tracking              
tracking = {
  -- velocity profile over the channel height
  -- In the Poisseuille flow, this should be parabolic and depend on the pressure drop
{ label = 'velocityProfile',
  variable = {'velocity'},
  shape= { kind = 'canoND', object={origin ={  0., -20., 0.}, vec = {  0.,40.,0.},
                                    segments = 4} },
  time_control = { min = {iter=tmax}}, -- dump at last timestep
  folder = 'tracking/',
  format = 'asciiSpatial'
  },
{ label = 'pressureDrop',
  variable = {'density'}, 
  shape= { kind = 'canoND', object={origin ={-75., 0., 0.}, vec = {150.,0.,0.}, 
                                    segments=100} },
  time_control = { min = {iter=tmax}},
  folder = 'tracking/',
  format = 'asciiSpatial'
  },
{ label = 'probePress', 
  variable = {'density'}, 
  shape= { kind = 'canoND', object={origin ={0., 0., 0.} } },
  time_control = { min = {iter=0}, max = {iter=tmax}, interval = {iter=20}},
  format = 'ascii'
  },
{ label = 'totalDens', 
  variable = {'density'}, 
  reduction = {'sum'}, 
  shape= { kind = 'all' },
  time_control = { min = {iter=0}, max = {iter=tmax}, interval = {iter=1}},
  folder = 'tracking/',
  format = 'ascii'
  },
  }

-- Restart settings
restart_deactivated = {
  -- If this table is available it will write a restart file
  -- with the specified options.
  write = 'restart/', -- the folder the restart files are written to
  time_control = {min = {iter=tmax}}
}
