--This file contains common parameters

level = tonumber(os.getenv('LEVEL'))
if level == nil then
  level = 6
end
Re = tonumber(os.getenv('Re'))
if Re == nil then
  Re = 30
end

if os.getenv('MULTI') == 'y' then
  multi_level = 1
else
  multi_level = 0
end

if os.getenv('SCALING') == 'acoustic' then
  diffusive = false
  scaling = 'acoustic'
else
  diffusive = true  -- diffusive or acoustic scaling?
  scaling = 'diffusive'
end

if os.getenv('INTP') == 'linear' then
  interpolation_method = 'linear'
else
  interpolation_method = 'quadratic'
end

identify_kind = os.getenv('KIND')
if identify_kind == nil then
  identify_kind = 'lbm_incomp'
end

simulation_name = 'channel2D'
identify_label  = 'L'..level
useMomentsBC = false
useVelocityInlet = true
usePeriodic = true -- true = 2D channel, false = 3D channel
printRuntimeInfo = false
if useMomentsBC then
  layout = 'd2q9'
else
  layout = 'd3q19'
end

mesh = 'mesh/' -- Mesh files location

length = 4.0
dx = length / 2.^level
height = length / 4
if useMomentsBC then
  if useVelocityInlet then
    inlet_kind = 'moments_vel'
  else
    inlet_kind = 'moments_press'
  end
  outlet_kind = 'moments_press'
  wall_kind   = 'moments_wall'
  height = height - dx
  top_y = height/2 + dx
  bottom_y = -height/2 - dx
else
  if useVelocityInlet then
    inlet_kind = 'inlet_ubb'
  else
    inlet_kind = 'outlet_expol'
  end
  outlet_kind = 'outlet_expol'
  wall_kind   = 'wall'
  top_y = height/2 + dx/2
  bottom_y = -height/2 - dx/2
end
ly = height / dx       -- number of elements for channel height
lx = length / dx - 2   -- number of elements for channel height
R  = height / 2
inlet_x  = -length/2 + dx/2
outlet_x =  length/2 - dx/2

-- For Grid convergence,
-- keep Re = U_max * height / nuPhy
if diffusive then
  -- For DIFFUSIVE scaling, keep omega constant, adjust velMax accordingly
  omega = 4./3.
  tmaxLB  =  5000 * ( 4.^(level - 5 ))
else
  -- For ACOUSTIC scaling, keep vel constant, adjust omega accordingly
  velMaxLB = 0.005
  omega = 2 * Re / ( 4.0 * velMax * ly + Re )
  tmaxLB  =  9000 * ( 2.^(level - 6 ))
end

-- Physical parameters
densityPhy = 1050
viscPhy = 0.0035
nuPhy = viscPhy / densityPhy

-- reference LB quantities
rho0 = 1.
cs2  = 1./3.
cs   = math.sqrt(cs2)
p0LB = rho0*cs2
tmax = tmaxLB
tau  = 1.0 / omega
nuLB = ( 1.0 / omega - 0.5 ) / 3.0   -- viscosity
dt   = nuLB * dx * dx / nuPhy
velMaxPhy = Re * nuPhy / height
velMaxLB  = velMaxPhy / dx * dt
mach = velMaxLB / cs

p0Phy = p0LB * densityPhy * dx * dx / dt / dt -- pressure at outlet
pOutlet = p0Phy
rhoLBOutlet = pOutlet * dt * dt / densityPhy / dx / dx / cs2
tmaxPhy = tmaxLB * dt

dp_dx = -2.0 * velMaxPhy * viscPhy / R / R
dp = dp_dx * ( length - 3*dx )
pInlet = pOutlet - dp

-- Analytical solutions
function wssA(x, y, z)
  return viscPhy * 2.0 * math.abs(y) * velMaxPhy / R / R
end
function strainRateA(x, y, z)
  return -1*(y) * velMaxPhy / R / R
end
function velA(x, y, z)
  return velMaxPhy * ( 1 - (y)^2 / R^2 )
end
-- outlet pressure is @ length/2 - dx
-- outlet STL is at length/2 - dx/2,
-- the outlet element is at length/2 - dx/2 - dx
function pressureOutlet(x,y,z)
  return pOutlet
end
-- pressure analytical
function pressureA(x,y,z)
  return pOutlet + dp_dx * ( x - (length/2-3*dx/2) )
end
