-- load lua variables from simParam.lua
require "common"
tracking_fol = 'tracking/'
scheme_kind = 'bgk'
bc_inlet = 'spc_inlet_ubb'
bc_inlet = 'spc_inlet_eq'
bc_inlet = 'spc_molefrac'
bc_inlet = 'spc_moments_molefrac'

bc_outlet = 'wall'
bc_outlet = 'spc_outlet_expol'
bc_outlet = 'spc_outlet_eq'
bc_outlet = 'spc_outlet_vel'
bc_outlet = 'spc_vel_bb'
bc_outlet = 'spc_moments_vel'

bc_mem = 'wall'
bc_mem = 'spc_blackbox_mem_ion'
bc_mem = 'spc_moments_blackbox_mem_ion'

-- simulation paramters
io_buffer_size = 1
printRuntimeInfo = true

---- communication pattern
--isend_irecv_overlap gathered_type isend_irecv typed_isend_irecv
commpattern = 'isend_irecv'
control_routine = 'fast'
-- Simulation name
simulation_name = 'taylorDispersion'
mesh = 'mesh/'

-- Interpolation method
-- average, copyfirst, linear, debug
interpolation_method = 'average'                   
scaling = 'diffusive'

ebug = {logging = {level=1, filename = 'dbg_bgk.out'},
         debugMode = true, debugFiles = true, verbose =  true, verboseLevel = 100}

-- Time step settings
interval = tmax_p/10
sim_control = {
  time_control = {
    max = tmax_p, 
    interval = interval 
  }
  --time_control = {
  --  max = {iter=100}, 
  --  interval = {iter=10} 
  --}
}

-- restart
restart = {
      ead = 'restart/taylorDispersion_lastHeader.lua',
      write = 'restart/',
      time_control = { min = 0, max = tmax_p, interval = tmax_p/10}
}

physics = { --rho0 = rho0_p, 
            dt = dt, 
            molWeight0 = m_min, 
            moleDens0 = moleDens0 }

-- scheme model for single fluid simulation
scheme = {
  identify = {
      label = 'mix',
      kind = 'multi-species_liquid', 
      relaxation = scheme_kind, 
      --relaxation = 'mrt', 
-- scheme layout
      layout = 'd3q19'
      --layout = 'd2q9'
  },

  physics = { moleDens0 = moleDens0, 
              dt = dt, 
              molWeight0 = m_min },

  external_force = { electric_fieldX = 0.0,
                     electric_fieldY = 1e-9,
                     electric_fieldZ = 0.0,
                     gravity_fieldX = 0.0,--deltaP, --kinematic pressure drop
                     gravity_fieldY = 0.0,
                     gravity_fieldZ = 0.0
                     }, 

  mixture = { rho0 = rho0_p, moleDens0 = moleDens0,
              omega_diff = omega_diff,
              kine_shear_viscosity = nu_phy,
              --bulk_viscosity = 1e-7,
              --omega_kine = 2.0,
              omega_hom = 0.0001,
              theta_eq = 1.0,
              temp = 293.15,
              electricField = { 0.0, 5e-9, 0.0},
              gravityField = { deltaP/rho0_p,0.0,0.0},
              initial_condition = { pressure = 0.0}--TGV_kinematicPressure }
            },
  -- field which defines fluid or specie
  -- Single fluid simulation
  field = {{
    label = 'H2O'
-- species properties
   ,species = { molweight = mH2O, 
                diff_coeff = { diff_diag, diff_H2O_Na, diff_H2O_Cl },
                charge_nr = charge_nr_H2O}
-- Initial condition
   ,initial_condition = { 
      molefraction = IC_H2O,
      velocityX = IC_velocity,
      velocityY = 0.0,
      velocityZ = 0.0
    }
   ,boundary_condition = {
     {
       label = 'west',
       kind = bc_inlet,
       molefraction = BC_H2O,
       velocityX = velocityX,
       velocityY = 0.0,
       velocityZ = 0.0,
       --order = 2
     },
     {
       label = 'east',
       kind = bc_outlet,
       velocityX = velocityX,
       velocityY = 0.0,
       velocityZ = 0.0,
       pressure = 0.0
     },
     { label = 'north', kind = 'wall'},
     { label = 'south', kind = 'wall'}
    }
  }
 ,{
    label = 'Na'
   ,species = { molweight = mNa, 
                diff_coeff = { diff_H2O_Na, diff_diag, diff_Na_Cl },
                charge_nr = charge_nr_Na}
   ,initial_condition = {
      molefraction = IC_Na,
 --     velocityX = 0.0,
      velocityX = IC_velocity,
      velocityY = 0.0,
      velocityZ = 0.0
    }
   ,boundary_condition = {
     {
       label = 'west',
       kind = bc_inlet,
       molefraction = BC_Na,
--       velocityX = 0.04,
       velocityX = velocityX,
       velocityY = 0.0,
       velocityZ = 0.0,
       --order = 2
     },
     {
       label = 'east',
       kind = bc_outlet,
       velocityX = velocityX,
       velocityY = 0.0,
       velocityZ = 0.0,
       pressure = 0.0
     },
     { label = 'north', 
       kind = bc_mem,
       flux_dir = {0,1,0}, 
       transference_number = 0.971 },
     { label = 'south', 
       kind = bc_mem,
       flux_dir = {0,1,0}, 
       transference_number = (1.0-0.998) },
    }
  }
 ,{
    label = 'Cl'
   ,species = { molweight = mCl, 
                diff_coeff = { diff_H2O_Cl, diff_Na_Cl, diff_diag },
                charge_nr = charge_nr_Cl}
   ,initial_condition = {
      molefraction = IC_Cl,
      velocityX = IC_velocity,
      velocityY = 0.0,
      velocityZ = 0.0
    }
   ,boundary_condition = {
     {
       label = 'west',
       kind = bc_inlet,
       molefraction = BC_Cl,
       velocityX = velocityX,
       velocityY = 0.0,
       velocityZ = 0.0,
       --order = 2
     },
     {
       label = 'east',
       kind =  bc_outlet,
       velocityX = velocityX,
       velocityY = 0.0,
       velocityZ = 0.0,
       pressure = 0.0
     },
     { label = 'north', 
       kind = bc_mem,
       flux_dir = {0,-1,0}, 
       transference_number = (1.0-0.971) },
     { label = 'south', 
       kind = bc_mem,
       flux_dir = {0,-1,0}, 
       transference_number = 0.998 },
    }
  }
 }

,racking = {
  {
    label = 'inlet_velMax',
    variable = {
                {'velMag_phy'},
                },
    folder=tracking_fol,            
    shape = {kind='canoND', object = { origin = {dx/2.,height/2.0,dx/2.}}
            },
    format='ascii',
    time_control={ min=0, max = tmax_p, interval = dt}
  }
  ,
  {
    label = 'inlet_numDens',
    variable = {{'H2O_numDensity_phy'},{'Na_numDensity_phy'},{'Cl_numDensity_phy'},
                {'numDensity_phy'},
                {'velocity_phy'},
                {'H2O_moleflux_phy'},{'Na_moleflux_phy'},{'Cl_moleflux_phy'}},
    folder=tracking_fol,            
    shape = {kind='canoND', object = { origin = {dx/2.,0.0,dx/2.},
                                       vec = {0.0,height,0.0},
                                       segments = nHeight+2}
            },
    format='ascii',
    reduction={'average','average','average','average','average','average','average','average'},
    time_control={ min=0, max = tmax_p, interval = tmax_p/tmax}
  }
  ,
  {
    label = 'outlet_numDens',
    variable = {{'H2O_numDensity_phy'},{'Na_numDensity_phy'},{'Cl_numDensity_phy'},
                {'numDensity_phy'},
                {'velocity_phy'},
                {'H2O_moleflux_phy'},{'Na_moleflux_phy'},{'Cl_moleflux_phy'}},
    folder=tracking_fol,            
    shape = {kind='canoND', object = { origin = {length-dx/2.,0.0,dx/2.},
                                       vec = {0.0,height,0.0},
                                       segments = nHeight+2}
            },
    format='ascii',
    reduction={'average','average','average','average','average','average','average','average'},
    time_control={ min=0, max = tmax_p, interval = dt}
  }
  ,
  {
    label = 'aem_numDens',
    variable = {{'H2O_numDensity_phy'},{'Na_numDensity_phy'},{'Cl_numDensity_phy'},
                {'numDensity_phy'},
                {'H2O_moleflux_phy'},{'Na_moleflux_phy'},{'Cl_moleflux_phy'}},
    folder=tracking_fol,            
    shape = {kind='canoND', object = { origin = {0.0,dx/2,dx/2.0},
                                       vec = {length,0.0,0.0},
                                       segments = nLength+2}
            },
    format='ascii',
    reduction={'average','average','average','average','average','average','average'},
    time_control={ min=0, max = tmax_p, interval = dt}
  }
  ,
  {
    label = 'cem_numDens',
    variable = {{'H2O_numDensity_phy'},{'Na_numDensity_phy'},{'Cl_numDensity_phy'},
                {'numDensity_phy'},
                {'H2O_moleflux_phy'},{'Na_moleflux_phy'},{'Cl_moleflux_phy'}},
    folder=tracking_fol,            
    shape = {kind='canoND', object = { origin = {0.0,height-dx/2.,dx/2.0},
                                       vec = {length,0.0,0.0},
                                       segments = nLength+2}
            },
    format='ascii',
    reduction={'average','average','average','average','average','average','average'},
    time_control={ min=0, max = tmax_p, interval = dt}
  }
  ,
  {
    label = 'tot_numDens',
    variable = {{'H2O_numDensity_phy'},{'Na_numDensity_phy'},{'Cl_numDensity_phy'},
                {'numDensity_phy'},
                },
    folder=tracking_fol,            
    shape = {kind='all'},
    format='ascii',
    reduction={'average','average','average','average'},
    time_control={ min=0, max = tmax_p, interval = dt}
  }

}
}

