require "common"

sim_control = {
  time_control = {
    max = { iter = tmax },
    interval = { iter = tmax/10 }
  },
}   -- sim_control

logging = {
  level = 1,
  --filename = simulation_name..'.log',
  root_only = false,
  real_form = 'EN5.3',
  int_form = 'I6',
} -- logging

estart = {
  read = 'restart/'..simulation_name..'_lastHeader.lua',
  --write = 'restart/',
  --time_control = {
  --  min = { iter = tmax },
  --  max = { iter = tmax },
  --  interval = { iter = 1 },
  --},
}

physics = {
  dt = dt,
  rho0 = densityPhy,
} -- physics table

scheme = {
  identify = {
    label = 'L_'..level..'_n_'..n..'_Re_'..Re,
    kind = 'lbm_nNwtn',
    relaxation = 'bgk_pl',
    layout = 'd3q19',
  },

  -- add variables for analytical solutions
  add_variable = {
    { name='wssPL', ncomponents=1, wssPL=wssPL },
    { name='velPL', ncomponents=1, velPL=velPL },
    { name='strainPL', ncomponents=1, strainPL=strainPL },
    { name='velNew', ncomponents=1, velNew=velA },
    { name='strainNew', ncomponents=1, strainNew=strainRateA },
    { name='wssNew', ncomponents=1, wssNew=wssA },
  },

  racking = {
    { -- whole fluid domain
      label = 'global',
      variable = {
        {'velocity_phy'},{'shearRate_phy'},{'wss_phy'}
      },
      shape = { kind = 'global', },
      time_control = { 
        min = { iter = 0 },
        max = { iter = tmax },
        interval = { iter = 1 }, },
      folder = './tracking/',
      format = 'harvester',
    },
  },
  tracking = {
    { -- spatial across channel at last iteration
      label = 'spatial',
      variable = {
        {'velocity_phy'},{'velPL'},
        {'strainRate_phy'},{'strainPL'},
        {'wss_phy'},{'wssPL'},
        {'omega'},
        {'shearRate_phy'},  -- 18
        -- Newtonian fluid profile, only for comparison
        {'velNew'}, {'strainNew'}, {'wssNew'}, -- 19, 20, 21
        {name = 'relDiff', ncomponents = 1, dep = {'velMag_phy','velPL'},},
      }, -- variable
      shape = {
        kind = 'canoND',
        object = {
          origin = { 0.0, -height/2, dx/2},
          vec = { 0.0, height, 0.0},
          segments = {200},
        }
      }, -- shape
      time_control = {
        min = { iter = tmax },
        max = { iter = tmax },
        interval = { iter = tmax },
      },
      folder = './tracking/',
      format = 'asciiSpatial',
    },
    { -- velocity l2norm
      label = 'vel_l2norm',
      variable = {
        {name = 'difference', ncomponents = 1, dep = {'velMag_phy','velPL'},},
        {'velPL'},
        {name = 'relDiff', ncomponents = 1, dep = {'velMag_phy','velPL'},},
      },
      shape = {
        kind = 'canoND',
        object = {
          origin = { 0.0, -height/2, dx/2},
          vec = { 0.0, height, 0.0},
          segments = {100}
        }
      },
      time_control = {
        min = { iter = 0 },
        max = { iter = tmax },
        interval = { iter = tmax / 10 },
      },
      reduction = {'l2norm', 'l2norm', 'l2norm'},
      folder = './tracking/', format = 'ascii',
    },
    { -- wss l2norm
      label = 'wss_l2norm',
      variable = {
        { name = 'difference', ncomponents = 1, dep = {'wss_phy','wssPL'},},
        {'wssPL'},
        { name = 'relDiff', ncomponents = 1, dep = {'wss_phy','wssPL'},},
      },
      shape = {
        kind = 'canoND',
        object = {
          origin = { 0.0, dx, dx/2 },
          vec = { 0.0, R, 0.0},
          segments = { 100 }
        }
      },
      time_control = {
        min = { iter = 0 },
        max = { iter = tmax },
        interval = { iter = tmax / 10 },
      },
      reduction = {'l2norm', 'l2norm', 'l2norm'},
      folder = './tracking/', format = 'ascii',
    },
    --{ -- results across channel over time
    --  label = 'over_time',
    --  variable = {
    --    --{'velocity_phy'},  {'velPL'},
    --    {'strainRate_phy'},{'strainPL'},
    --    --{'wss_phy'},       {'wssPL'},
    --  },
    --  shape = {
    --    kind = 'canoND',
    --    object = {
    --      origin = { 0.0, -height/2, dx/2},
    --      vec = { 0.0, height, 0.0},
    --      segments = {200},
    --    }
    --  }, -- shape
    --  time_control = {
    --    min = { iter = 0 },
    --    max = { iter = tmax },
    --    interval = { iter = tmax / 1000 },
    --  },
    --  folder = './tracking/',
    --  format = 'ascii',
    --},
  }, -- tracking

  fluid = {
    omega = omega,
    rho0 = densityPhy,
    nonNewtonian = {
      model = 'power_law',
      n = n,
      k = viscPhy,
    },
  },

  initial_condition = {
    pressure = p0Phy,
    velocityX = velPL, velocityY = 0.0, velocityZ = 0.0,
    Sxx = 0, Syy = 0, Szz = 0, Sxy = strainRatePL, Syz = 0, Sxz = 0,
    omega = omega,
  },

  boundary_condition = {
    { label = 'outlet',kind = 'outlet_expol', pressure = p0Phy, }, -- outlet
    --{ label = 'inlet', kind = 'outlet_expol', pressure = p0Phy+dp, }, -- inlet
    { label = 'top_wall', kind = 'wall'},
    { label = 'bottom_wall', kind = 'wall'},
    {
      label = 'inlet',
      kind = 'inlet_ubb',
      velocityX = {
        kind = 'combined',
        transient= {
          predefined = 'smooth',
          min_factor = velMaxPhy,
          max_factor = velMaxPhy,
          from_time  = 0,
          to_time    = tmaxPhy/4,
        },
        spatial = {
          predefined='parabol',
          shape = {
            kind = 'canoND',
            object = {
              origin = { -length/2+dx, -R, dx/2},
              vec = { 0.0, 2*R, 0.0 }
            }
          } -- shape table
        } -- spatial table
      }, -- velX table
      velocityY = 0.0,
      velocityZ = 0.0,
    } -- inlet table
  }, -- boundary table
} -- scheme table

ebug = { 
  debugMode = true,  -- activate global debug modus
  debugFiles = true, -- open the debugFiles for each process dbgOut0000**.out
  --dumpDependencies = true, -- write the dependencies between levels to disk
  dumpAuxLists = true,
  logging = {
    level = 11, 
    filename = 'dbg', 
    root_only = false 
  },
}

