module tem_precice_module
  use, intrinsic :: iso_c_binding
  use env_module,                only: rk, labelLen
  use aotus_module,              only: flu_State, aot_get_val, &
                                 &     aoterr_NonExistent, aoterr_fatal
  use aot_table_module,          only: aot_table_open, aot_table_close
  use aot_table_ops_module,      only: aot_table_length 
  use tem_logging_module,        only: logUnit
  use tem_aux_module,            only: tem_abort, tem_open_distconf
  use tem_tools_module,          only: upper_to_lower
  use tem_time_module,           only: tem_time_type

  implicit none

  private


  logical , parameter :: precice_available= .false.

  type tem_precice_type
    ! general info for precice
    ! name of solver, should be indetical to name in precice_configeFile
    character(len=labelLen) :: accessor
    character(len=labelLen) :: precice_configFile
    !> for initialize data for precice
    integer                :: action_isrequired
  end type tem_precice_type

  type(tem_precice_type) :: precice

  interface tem_spacetime_precice_for
    module procedure tem_spacetime_precice_scalar
    module procedure tem_spacetime_precice_vector
  end interface tem_spacetime_precice_for

  interface tem_precice_write
    module procedure tem_precice_write_scalar
    module procedure tem_precice_write_vector
  end interface tem_precice_write

  public :: precice
  public :: precice_available
  public :: tem_precice_load, tem_precice_init, tem_precice_create
  public :: tem_precice_advance
  public :: tem_precice_finalize
  public :: tem_precice_ongoing
  public :: tem_spacetime_precice_for
  public :: tem_precice_write
  public :: tem_precice_set_vertex_pos
  public :: tem_precice_action_required
  public :: tem_precice_action_write_initial_data
  public :: tem_precice_action_write_iter_checkp
  public :: tem_precice_action_read_iter_checkp
  public :: tem_precice_initialize_data
  public :: tem_precice_fulfilled_Action  
  public :: tem_precice_initmeshID
  public :: tem_precice_init_readDataID
  public :: tem_precice_init_writeDataID

contains


  !*****************************************************************************
  subroutine tem_precice_load(conf)
    !---------------------------------------------------------------------------
    !> The filename to read the configuration from.
    type(flu_state), intent(in) :: conf
    !---------------------------------------------------------------------------

  end subroutine tem_precice_load 
  !*****************************************************************************

  !*****************************************************************************
  subroutine tem_precice_create(rank,comm_size)
    !---------------------------------------------------------------------------
    integer, intent(in) :: rank
    integer, intent(in) :: comm_size
    !---------------------------------------------------------------------------
  
  end subroutine tem_precice_create
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_init_readDataID(n_readData, read_dataName, read_dataID,&
    &                                   meshID)
    !---------------------------------------------------------------------------
    integer, intent(in)                    :: n_readData
    character(len=labelLen), intent(in)    :: read_dataName(:)
    integer, intent(inout)                 :: read_dataID(:)
    integer, intent(in)                    :: meshID
    !---------------------------------------------------------------------------
    character(len=labelLen, kind=c_char) :: c_dataName
    integer(kind=c_int)                  :: c_dataNameLength
    integer(kind=c_int)                  :: c_meshID
    integer(kind=c_int)                  :: c_dataID
    integer                              :: i
    !---------------------------------------------------------------------------
  end subroutine tem_precice_init_readDataID
  !*****************************************************************************
    

  !*****************************************************************************
  subroutine tem_precice_init_writeDataID( n_writeData, write_dataName, &
    &                                      write_dataID, meshID         )
    !---------------------------------------------------------------------------
    integer, intent(in)                    :: n_writeData
    character(len=labelLen), intent(in)    :: write_dataName(:)
    integer, intent(inout)                 :: write_dataID(:)
    integer, intent(in)                    :: meshID
    !---------------------------------------------------------------------------
    !---------------------------------------------------------------------------
  end subroutine tem_precice_init_writeDataID
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_initmeshID(meshName, meshID)
    !---------------------------------------------------------------------------
    character(len=labelLen) :: meshName
    integer                 :: meshID
    !---------------------------------------------------------------------------

  end subroutine tem_precice_initmeshID
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_init(timesteplimit)
    !---------------------------------------------------------------------------
    real(kind=rk), intent(inout) :: timestepLimit
    real(kind=c_double)    :: c_timestepLimit
    !---------------------------------------------------------------------------

  end subroutine tem_precice_init
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_initialize_data()
    !---------------------------------------------------------------------------

  end subroutine tem_precice_initialize_data

  subroutine tem_precice_advance(timestepLimit)
    !---------------------------------------------------------------------------
    real(kind=rk), intent(inout) :: timestepLimit
    real(kind=c_double) :: c_timestepLimit
    !---------------------------------------------------------------------------
  end subroutine tem_precice_advance
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_finalize()
    !---------------------------------------------------------------------------
  end subroutine tem_precice_finalize
  !*****************************************************************************
 

  !*****************************************************************************
  subroutine tem_precice_ongoing(isOngoing)
    !---------------------------------------------------------------------------
    integer, intent(inout)   :: isOngoing
    !---------------------------------------------------------------------------
  end subroutine tem_precice_ongoing
  !*****************************************************************************

  
  !*****************************************************************************
  subroutine tem_precice_action_write_initial_data(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(inout) :: nameAction
    character(len=labelLen, kind=c_char) :: c_nameAction
    integer(kind=c_int) :: c_lengthNameAction
    !---------------------------------------------------------------------------
  end subroutine tem_precice_action_write_initial_data
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_action_write_iter_checkp(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(inout) :: nameAction
    !---------------------------------------------------------------------------
  end subroutine tem_precice_action_write_iter_checkp
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_action_read_iter_checkp(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(inout) :: nameAction
    !---------------------------------------------------------------------------
  end subroutine tem_precice_action_read_iter_checkp
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_fulfilled_Action(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(in) :: nameAction
    character(len=labelLen, kind=c_char) :: c_nameAction
    integer(kind=c_int) :: c_lengthNameAction
    !---------------------------------------------------------------------------
  end subroutine tem_precice_fulfilled_Action
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_action_required(NameAction, isRequired)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(in) :: NameAction
    integer, intent(out) :: isRequired
  end subroutine tem_precice_action_required
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_set_read_pos(meshID ,position, positionID )
    !---------------------------------------------------------------------------
    integer, intent(in) :: meshID
    real(kind=rk), intent(in) :: position(3)
    integer, intent(out) :: positionID  
    !---------------------------------------------------------------------------
  endsubroutine tem_precice_set_read_pos
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_set_vertex_pos(meshID, meshsize, position, positionID )
    !---------------------------------------------------------------------------
    integer, intent(in) :: meshID
    integer, intent(in) :: meshsize
    real(kind=rk), intent(in) :: position(meshsize*3)
    integer, intent(out) :: positionID(meshsize)  
    !---------------------------------------------------------------------------
  endsubroutine tem_precice_set_vertex_pos
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_set_write_pos(meshID, position, positionID )
    !---------------------------------------------------------------------------
    integer, intent(in) :: meshID
    real(kind=rk), intent(in) :: position(3)
    integer, intent(out) :: positionID  
    !---------------------------------------------------------------------------
  endsubroutine tem_precice_set_write_pos
  !*****************************************************************************


  !*****************************************************************************
  function tem_spacetime_precice_scalar(dataID, posID, npoints) result(res)
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: npoints
    real(kind=rk) :: res(npoints) !< return value
    !------------------------------------------------------------------------
  end function tem_spacetime_precice_scalar
  !*****************************************************************************


  !*****************************************************************************
  function tem_spacetime_precice_vector(dataID, posID, npoints, nscalar, vector) &
    &                                   result(res)
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: nscalar
    integer, intent(in)          :: npoints
    logical, intent(in)          :: vector
    real(kind=rk) :: res(npoints,nscalar) !< return value
    !------------------------------------------------------------------------
  end function tem_spacetime_precice_vector
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_write_scalar(dataID, posID, npoints, val) 
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(npoints)
    integer, intent(in)          :: npoints
    real(kind=rk), intent(in)    :: val(npoints)
    !------------------------------------------------------------------------
  end subroutine tem_precice_write_scalar
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_write_vector(dataID, posID, npoints, nscalar,val) 
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: npoints
    integer, intent(in)          :: nscalar
    real(kind=rk), intent(in)    :: val(nscalar,npoints)
    !------------------------------------------------------------------------
  end subroutine tem_precice_write_vector
  !*****************************************************************************

end module tem_precice_module
