module tem_precice_module
  use, intrinsic :: iso_c_binding
  use env_module,                only: rk, labelLen
  use aotus_module,              only: flu_State, aot_get_val, &
                                 &     aoterr_NonExistent, aoterr_fatal
  use aot_table_module,          only: aot_table_open, aot_table_close
  use aot_table_ops_module,      only: aot_table_length 
  use tem_logging_module,        only: logUnit
  use tem_aux_module,            only: tem_abort, tem_open_distconf
  use tem_tools_module,          only: upper_to_lower
  use tem_time_module,           only: tem_time_type
  use PreCICE_consts_module,     only: precicef_action_write_initial_data
  use PreCICE_solver_if_module,  only: precicef_create, &
    &                                  precicef_initialize, &
    &                                  precicef_initialize_data, &
    &                                  precicef_advance, &
    &                                  precicef_finalize, &
    &                                  precicef_set_vertex, &
    &                                  precicef_set_vertices, &
    &                                  precicef_ongoing, &
    &                                  precicef_get_data_id, &
    &                                  precicef_read_sdata, &
    &                                  precicef_read_bsdata, &
    &                                  precicef_read_vdata, &
    &                                  precicef_write_sdata, &
    &                                  precicef_write_bsdata, &
    &                                  precicef_write_vdata, &
    &                                  precicef_get_mesh_id,&
    &                                  precicef_action_required, &
    &                                  precicef_fulfilled_action
  use PreCICE_consts_module,     only: precicef_action_write_initial_data, &
    &                                  precicef_action_write_iter_checkp, &
    &                                  precicef_action_read_iter_checkp

  implicit none

  private


  logical , parameter :: precice_available= .true.

  type tem_precice_type
    ! general info for precice
    ! name of solver, should be indetical to name in precice_configeFile
    character(len=labelLen) :: accessor
    character(len=labelLen) :: precice_configFile
    !> for initialize data for precice
    integer                :: action_isrequired
  end type tem_precice_type

  type(tem_precice_type) :: precice

  interface tem_spacetime_precice_for
    module procedure tem_spacetime_precice_scalar
    module procedure tem_spacetime_precice_vector
  end interface tem_spacetime_precice_for

  interface tem_precice_write
    module procedure tem_precice_write_scalar
    module procedure tem_precice_write_vector
  end interface tem_precice_write

  public :: precice
  public :: precice_available
  public :: tem_precice_load, tem_precice_init, tem_precice_create
  public :: tem_precice_advance
  public :: tem_precice_finalize
  public :: tem_precice_ongoing
  public :: tem_spacetime_precice_for
  public :: tem_precice_write
  public :: tem_precice_set_vertex_pos
  public :: tem_precice_action_required
  public :: tem_precice_action_write_initial_data
  public :: tem_precice_action_write_iter_checkp
  public :: tem_precice_action_read_iter_checkp
  public :: tem_precice_initialize_data
  public :: tem_precice_fulfilled_Action  
  public :: tem_precice_initmeshID
  public :: tem_precice_init_readDataID
  public :: tem_precice_init_writeDataID

contains


  !*****************************************************************************
  subroutine tem_precice_load(conf)
    !---------------------------------------------------------------------------
    !> The filename to read the configuration from.
    type(flu_state), intent(in) :: conf
    !---------------------------------------------------------------------------
    integer                     :: thandle
    integer                     :: sub_thandle
    integer                     :: iError
    character(len=labelLen)     :: accessor
    character(len=labelLen)     :: precice_configFile
    integer                     :: i
    !---------------------------------------------------------------------------

    ! get the name of the solver and the config file from the lua script
    call aot_table_open(L = conf,  thandle = thandle, &
      &                 key = 'precice')

    if (thandle /= 0) then
      call aot_get_val(L = conf, thandle = thandle, key = 'accessor', &
        &              val = precice%accessor, default = 'Ateles',    &
        &              ErrCode = iError                               )
      call aot_get_val(L = conf, thandle = thandle, key = 'configFile', &
        &              val = precice%precice_configFile,                &
        &              default = 'config.xml', ErrCode = iError         )

      write(logUnit(1),*) '* preCICE asseccor = "'//trim(precice%accessor)//'"'
      write(logUnit(1),*) '* preCICE configFile = "'//trim(precice%precice_configFile)//'"' 

    else ! no precice table
      write(logUnit(1),*) 'ERROR while init preCICE:' 
      write(logUnit(1),*) '      No Precice table is provided' 
      write(logUnit(1),*) '      Stopping...'
      call tem_abort()
    end if

    call aot_table_close(L = conf, thandle = thandle)
  end subroutine tem_precice_load 
  !*****************************************************************************

  !*****************************************************************************
  subroutine tem_precice_create(rank,comm_size)
    !---------------------------------------------------------------------------
    integer, intent(in) :: rank
    integer, intent(in) :: comm_size
    !---------------------------------------------------------------------------
    character(len=labelLen, kind=c_char) :: c_accessor
    character(len=labelLen, kind=c_char) :: c_precice_configFile
    integer(kind=c_int) :: c_my_rank
    integer(kind=c_int) :: c_my_size
    integer(kind=c_int) :: c_accessorNameLength
    integer(kind=c_int) :: c_configFileNameLength
    integer                :: i
    !---------------------------------------------------------------------------
 
    c_accessor = precice%accessor
    c_precice_configFile = precice%precice_configFile
    c_my_rank = int(rank, kind=c_int)
    c_my_size = int(comm_size, kind=c_int)
    c_accessorNameLength = len_trim(precice%accessor)
    c_configFileNameLength = len_trim(precice%precice_configFile)

    call precicef_create(accessorName = c_accessor, &
      &                  configFileName = c_precice_configFile, &
      &                  solverProcessIndex = c_my_rank, &
      &                  solverProcessSize = c_my_size, &
      &                  lengthAccessorName = c_accessorNameLength, &
      &                  lengthConfigFileName = c_configFileNameLength)

  end subroutine tem_precice_create
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_init_readDataID(n_readData, read_dataName, read_dataID,&
    &                                   meshID)
    !---------------------------------------------------------------------------
    integer, intent(in)                    :: n_readData
    character(len=labelLen), intent(in)    :: read_dataName(:)
    integer, intent(inout)                 :: read_dataID(:)
    integer, intent(in)                    :: meshID
    !---------------------------------------------------------------------------
    character(len=labelLen, kind=c_char) :: c_dataName
    integer(kind=c_int)                  :: c_dataNameLength
    integer(kind=c_int)                  :: c_meshID
    integer(kind=c_int)                  :: c_dataID
    integer                              :: i
    !---------------------------------------------------------------------------
    c_meshID = meshID
    do i= 1, n_readData
      c_dataNameLength = len_trim(read_dataName(i))
      c_dataName = read_dataName(i)
      call precicef_get_data_id(dataName = c_dataName,           &
        &                       meshID   = c_meshID,             &
        &                       dataID   = c_dataID,             &
        &                       lengthDataName = c_dataNameLength)
      read_dataID(i) = c_dataID 
    end do
  end subroutine tem_precice_init_readDataID
  !*****************************************************************************
    

  !*****************************************************************************
  subroutine tem_precice_init_writeDataID( n_writeData, write_dataName, &
    &                                      write_dataID, meshID         )
    !---------------------------------------------------------------------------
    integer, intent(in)                    :: n_writeData
    character(len=labelLen), intent(in)    :: write_dataName(:)
    integer, intent(inout)                 :: write_dataID(:)
    integer, intent(in)                    :: meshID
    !---------------------------------------------------------------------------
    character(len=labelLen, kind=c_char) :: c_dataName
    integer(kind=c_int)                  :: c_dataNameLength
    integer(kind=c_int)                  :: c_dataID
    integer(kind=c_int)                  :: c_meshID
    integer                              :: i
    !---------------------------------------------------------------------------
    c_meshID = meshID
    !init write data_id
    do i= 1, n_writeData
      c_dataNameLength = len_trim(write_dataName(i))
      c_dataName = write_dataName(i)
      call precicef_get_data_id(dataName = c_dataName, &
        &                       meshID   = c_meshID,   &
        &                       dataID   = c_dataID,   &
        &                       lengthDataName = c_dataNameLength)
      write_dataID(i) = c_dataID 
    end do
  end subroutine tem_precice_init_writeDataID
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_initmeshID(meshName, meshID)
    !---------------------------------------------------------------------------
    character(len=labelLen) :: meshName
    integer                 :: meshID
    !---------------------------------------------------------------------------
    character(len=labelLen, kind=c_char) :: c_MeshName
    integer(kind=c_int)    :: c_geometryNameLength
    integer(kind=c_int)    :: c_meshID
    !---------------------------------------------------------------------------
    ! init the meshID
    c_geometryNameLength = len_trim(meshName)
    c_meshName = meshName

    call precicef_get_mesh_id(geometryName = c_meshName,               &
      &                       meshID = c_meshID,                       &
      &                       lengthGeometryName = c_geometryNameLength) 
    meshID = c_meshID

  end subroutine tem_precice_initmeshID
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_init(timesteplimit)
    !---------------------------------------------------------------------------
    real(kind=rk), intent(inout) :: timestepLimit
    real(kind=c_double)    :: c_timestepLimit
    !---------------------------------------------------------------------------

    c_timestepLimit = timesteplimit
    call precicef_initialize(c_timestepLimit)
    timestepLimit=c_timestepLimit
  end subroutine tem_precice_init
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_initialize_data()
    !---------------------------------------------------------------------------
    !---------------------------------------------------------------------------

    write(logUnit(1),*) 'Exchange initialize data in preCICE' 
    call precicef_initialize_data()

  end subroutine tem_precice_initialize_data
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_advance(timestepLimit)
    !---------------------------------------------------------------------------
    real(kind=rk), intent(inout) :: timestepLimit
    real(kind=c_double) :: c_timestepLimit
    !---------------------------------------------------------------------------

    c_timestepLimit = real(timestepLimit, kind=c_double)
    call precicef_advance(c_timestepLimit)
    ! do i need to convert the timestepLengthlimit from kind=c_double to kind=rk ??
    timestepLimit=c_timestepLimit
  end subroutine tem_precice_advance
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_finalize()
    call precicef_finalize()
  end subroutine tem_precice_finalize
  !*****************************************************************************
 

  !*****************************************************************************
  subroutine tem_precice_ongoing(isOngoing)
    !---------------------------------------------------------------------------
    integer, intent(inout)   :: isOngoing
    integer(kind=c_int) ::c_isOngoing  
    !---------------------------------------------------------------------------

    c_isOngoing = isOngoing
    call precicef_ongoing(c_isOngoing)
    isOngoing = c_isOngoing

  end subroutine tem_precice_ongoing
  !*****************************************************************************

  
  !*****************************************************************************
  subroutine tem_precice_action_write_initial_data(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(inout) :: nameAction
    character(len=labelLen, kind=c_char) :: c_nameAction
    integer(kind=c_int) :: c_lengthNameAction
    !---------------------------------------------------------------------------
    c_lengthNAmeAction = labelLen
    c_nameAction=''
    call precicef_action_write_initial_data(c_nameAction, c_lengthNameAction)
    NameAction = c_nameAction
  end subroutine tem_precice_action_write_initial_data
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_action_write_iter_checkp(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(inout) :: nameAction
    character(len=labelLen, kind=c_char) :: c_nameAction
    integer(kind=c_int) :: c_lengthNameAction
    !---------------------------------------------------------------------------
    c_lengthNAmeAction = labelLen
    c_nameAction=''
    call precicef_action_write_iter_checkp(c_nameAction, c_lengthNameAction)
    NameAction = c_nameAction
  end subroutine tem_precice_action_write_iter_checkp
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_action_read_iter_checkp(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(inout) :: nameAction
    character(len=labelLen, kind=c_char) :: c_nameAction
    integer(kind=c_int) :: c_lengthNameAction
    !---------------------------------------------------------------------------
    c_lengthNAmeAction = labelLen
    c_nameAction=''
    call precicef_action_read_iter_checkp(c_nameAction, c_lengthNameAction)
    NameAction = c_nameAction
  end subroutine tem_precice_action_read_iter_checkp
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_fulfilled_Action(nameAction)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(in) :: nameAction
    character(len=labelLen, kind=c_char) :: c_nameAction
    integer(kind=c_int) :: c_lengthNameAction
    !---------------------------------------------------------------------------
    c_NameAction = nameAction
    c_LengthNameAction = len_trim(nameAction)

    call precicef_fulfilled_Action(c_nameAction, c_lengthNameAction)
  end subroutine tem_precice_fulfilled_Action
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_action_required(NameAction, isRequired)
    !---------------------------------------------------------------------------
    character(len=labelLen), intent(in) :: NameAction
    integer, intent(out) :: isRequired

    character(len=labelLen, kind=c_char) :: c_NameAction
    integer(kind=c_int) :: c_isRequired
    integer(kind=c_int) :: c_lengthAction
    !---------------------------------------------------------------------------
    
    c_nameaction = nameAction
    c_lengthAction = len_trim(NameAction)

    call precicef_action_required(action = c_nameaction,&
      &                           isRequired = c_isRequired,&
      &                           lengthAction = c_lengthAction)

    isRequired = c_isRequired

  end subroutine tem_precice_action_required
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_set_vertex_pos(meshID, meshsize, position, positionID )
    !---------------------------------------------------------------------------
    integer, intent(in) :: meshID
    integer, intent(in) :: meshsize
    real(kind=rk), intent(in) :: position(meshsize*3)
    integer, intent(out) :: positionID(meshsize)  
    !---------------------------------------------------------------------------
    integer(kind=c_int) :: c_meshID
    integer(kind=c_int) :: c_meshsize
    real(kind=c_double) :: c_pos(meshsize*3)
    integer(kind=c_int) :: c_posID(meshsize)  
    !---------------------------------------------------------------------------

    c_meshID = meshID
    c_meshsize = meshsize
    c_pos = real(position, kind=c_double)             
    call  precicef_set_vertices(meshID = c_meshID,     &
      &                         meshsize = c_meshsize, &
      &                         positions = c_pos,     &
      &                         positionIDs = c_posID  ) 
    positionID= int(c_posID) 

  endsubroutine tem_precice_set_vertex_pos
  !*****************************************************************************


  !*****************************************************************************
  function tem_spacetime_precice_scalar(dataID, posID, npoints) result(res)
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: npoints
    real(kind=rk) :: res(npoints) !< return value
    !------------------------------------------------------------------------
    integer(kind=c_int) :: c_isAvailable 
    integer             :: isAvailable 
    integer(kind=c_int) :: c_dataID 
    integer(kind=c_int) :: c_blocksize 
    integer(kind=c_int) :: c_dataNameLength
    integer(kind=c_int) :: c_positionID(npoints)
    real(kind=c_double) :: c_datavalue(npoints)
    !------------------------------------------------------------------------
    integer :: ipoint
    !------------------------------------------------------------------------

    
    call precicef_read_data_available(c_isAvailable)
    isAvailable = c_isAvailable
    if (isAvailable .eq. 1) then
!       do ipoint= 1, npoints  
       
         ! get and  convert int PosId to int_c Posid
         c_positionID= int(posID, kind=c_int)
         c_blocksize= int(npoints, kind=c_int)
         ! get the data id from the dataid array (pressure,velocity)
         c_dataID= int(dataID, kind=c_int)
         call precicef_read_bsdata( dataID = c_dataID, &
           &                        blocksize = c_blocksize,          &
           &                        valueIndices = c_positionID,      &
           &                        values = c_datavalue)

         res=c_dataValue

!       end do
    end if

  end function tem_spacetime_precice_scalar
  !*****************************************************************************


  !*****************************************************************************
  function tem_spacetime_precice_vector(dataID, posID, npoints, nscalar, vector) &
    &                                   result(res)
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: nscalar
    integer, intent(in)          :: npoints
    logical, intent(in)          :: vector
    real(kind=rk) :: res(npoints,nscalar) !< return value
    !------------------------------------------------------------------------
    integer(kind=c_int) :: c_dataID 
    integer(kind=c_int) :: c_positionID 
    real(kind=c_double) :: c_datavalue(nscalar)
    !------------------------------------------------------------------------
    integer :: ipoint
    !------------------------------------------------------------------------
    do ipoint= 1, npoints  
    
      ! get and  convert int PosId to int_c Posid
      c_positionID= int(posID(iPoint), kind=c_int)
      ! get the data id from the dataid array (pressure,velocity)
      c_dataID= int(dataID, kind=c_int)

      call precicef_read_vdata( dataID = c_dataID, &
        &                       valueIndex = c_positionID, &
        &                       dataValue = c_datavalue)

      res(iPoint, :)=c_dataValue(:)

    end do

  end function tem_spacetime_precice_vector
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_write_scalar(dataID, posID, npoints, val) 
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(npoints)
    integer, intent(in)          :: npoints
    real(kind=rk), intent(in)    :: val(npoints)
    !------------------------------------------------------------------------
    integer(kind=c_int) :: c_dataID 
    integer(kind=c_int) :: c_blocksize 
    integer(kind=c_int) :: c_dataName
    integer(kind=c_int) :: c_positionID(npoints) 
    real(kind=c_double) :: c_datavalue(npoints)
    !------------------------------------------------------------------------
    integer :: ipoint
    !------------------------------------------------------------------------

!    do ipoint= 1, npoints  
      
      ! get and  convert int PosId to int_c Posid
      c_positionID(:)= int(posID, kind=c_int)
      ! get the data id from the dataid array (pressure,velocity)
      c_dataID= int(dataID, kind=c_int)
      c_blocksize= int(npoints, kind=c_int)

      ! get and convert the value
      c_datavalue(:) = real(val, kind=c_double)

      call precicef_write_bsdata( dataID = c_dataID,           &
        &                         blockSize = c_blocksize,         & 
        &                         valueIndices = c_positionID, &
        &                         values = c_datavalue         )
     
!    end do

  end subroutine tem_precice_write_scalar
  !*****************************************************************************


  !*****************************************************************************
  subroutine tem_precice_write_vector(dataID, posID, npoints, nscalar,val) 
    !------------------------------------------------------------------------
    integer, intent(in)          :: dataID
    integer, intent(in)          :: posID(:)
    integer, intent(in)          :: npoints
    integer, intent(in)          :: nscalar
    real(kind=rk), intent(in)    :: val(nscalar,npoints)
    !------------------------------------------------------------------------
    integer(kind=c_int) :: c_dataID 
    integer(kind=c_int) :: c_dataName
    integer(kind=c_int) :: c_positionID 
    real(kind=c_double) :: c_datavalue(nscalar)
    !------------------------------------------------------------------------
    integer :: ipoint
    !------------------------------------------------------------------------

    do ipoint= 1, npoints  
      
      ! get and  convert int PosId to int_c Posid
      c_positionID= int(posID(iPoint), kind=c_int)
      ! get the data id from the dataid array (pressure,velocity)
      c_dataID= int(dataID, kind=c_int)

      ! convert the value
      c_datavalue(:) = real(val(:,ipoint), kind=c_double)

      call precicef_write_vdata( dataID = c_dataID, &
        &                        valueIndex = c_positionID, &
        &                        dataValue = c_datavalue(:) )
  
    end do

  end subroutine tem_precice_write_vector
  !*****************************************************************************

end module tem_precice_module
