! See copyright notice in the COPYRIGHT file.
program tem_tracking_test
  use mpi
  use env_module,            only: newunit, solSpecLen
  use tem_tracking_module,    only: tem_tracking_type, tem_trackingControl_type,&
    &                               tem_load_tracking
  use tem_var_system_module, only: tem_var_sys_dump
  use tem_logging_module, only: logUnit

  use aotus_module,    only: flu_State, aot_get_val, close_config,              &
    &                        open_config_chunk, open_config_file
  use aot_table_module, only: aot_table_open, aot_table_close,                  &
    &                         aot_table_length, aot_get_val
  use aot_out_module,   only: aot_out_type, aot_out_open, aot_out_close,        &
    &                         aot_out_open_table, aot_out_close_table, aot_out_val
  use tem_solveData_module, only: tem_solveData_type, tem_solveData_load


  implicit none

  integer :: varunit, trackunit
  type(tem_tracking_type), allocatable :: track(:)
  type(tem_trackingControl_type) :: tCtrl
  type(aot_out_type) :: out_conf
  type(tem_solveData_type) :: solSpec
  integer :: iError

  call MPI_Init(iError)
  trackunit = newUnit()
  open(unit=trackunit, status='scratch')
  call aot_out_open( put_conf = out_conf, outunit = trackunit)
  call aot_out_open_table( put_conf = out_conf, tname = 'tracking' )
  call aot_out_val( put_conf = out_conf, vname = 'label', val = 'point' )

  call aot_out_open_table( put_conf = out_conf, tname = 'variable' )
  call aot_out_open_table( put_conf = out_conf )
  call aot_out_val( put_conf = out_conf, val = 'density' )
  call aot_out_close_table( put_conf = out_conf )
  call aot_out_open_table( put_conf = out_conf )
  call aot_out_val( put_conf = out_conf, val = 'velocity' )
  call aot_out_close_table( put_conf = out_conf )
  call aot_out_close_table( put_conf = out_conf )

  call aot_out_val( put_conf = out_conf, vname = 'folder', val = 'tracking_' )

  call aot_out_open_table( put_conf = out_conf, tname = 'shape' )
  call aot_out_val( put_conf = out_conf, vname = 'kind', val = 'all' )
  call aot_out_close_table( put_conf = out_conf )

  call aot_out_val( put_conf = out_conf, vname = 'format', val = 'ascii' )

  call aot_out_open_table( put_conf = out_conf, tname = 'time' )
  call aot_out_val( put_conf = out_conf, vname = 'min', val = '1' )
  call aot_out_val( put_conf = out_conf, vname = 'max', val = '10' )
  call aot_out_val( put_conf = out_conf, vname = 'interval', val = '1' )
  call aot_out_close_table( put_conf = out_conf )

  call aot_out_close_table( put_conf = out_conf )


  call tem_solveData_load(me = solspec, file_unit = trackunit, &
    &                     rank = 0, comm = MPI_COMM_WORLD)
  close(trackunit)

  ! load tracking
  call tem_load_tracking( tCtrl          = tCtrl,       &
    &                     me             = track,       &
    &                     conf           = solspec%conf )
 
  !dump variable system
  varunit = newunit()
  call tem_var_sys_dump( sys=track(1)%header%requestedSys, outunit  = varunit ) 

  call close_config(L=solspec%conf)
  write(*,*) 'PASSED'

  call MPI_Finalize(iError)

end program tem_tracking_test
