! This utest program tests the moment conversion matrix
! A vector of pdf is converted to moments, then density and the first order
! moments are checked. And then the moments are converted back to pdf and are
! checked with original pdf vector.
program mus_moments_matrix_test
  use env_module,         only: rk, eps
  use tem_general_module, only: tem_start, tem_finalize, tem_general_type

  use mus_moments_module,       only: mus_init_moments
  use mus_moments_type_module,  only: mus_moment_type
  use mus_scheme_layout_module, only: mus_scheme_layout_type, &
    &                                 mus_init_stencil, mus_define_d3q19, &
    &                                 mus_finish_stencil

  implicit none

  logical :: error
  real(kind=rk) :: tolerance, max_error

  type( tem_general_type ) :: general
  type( mus_scheme_layout_type ) :: layout
  type( mus_moment_type ) :: moment
  real(kind=rk), allocatable :: valOldPDF(:)
  real(kind=rk), allocatable :: valNewPDF(:)
  real(kind=rk), allocatable :: valMoments(:)
  real(kind=rk) :: mx, my, mz, rho
  integer :: iVal

  call tem_start( 'Moments matrix utest', 'utest', general )
  error = .false.
  tolerance = eps * 2500._rk
  write(*,*) 'tolerance = ', tolerance

  ! Initialize stencil
  write(*,*) "Initialize stencil"
  call mus_init_stencil( stencil = layout%stencil, nStencils = 1 )
  call mus_define_d3q19( layout = layout, nElems = 1 )
  call mus_finish_stencil( layout )

  ! initialize moment type
  write(*,*) "Initialize moment matrix"
  call mus_init_moments( moment, layout, .true. )

  allocate( valOldPdf( layout%stencil(1)%QQ ) )
  allocate( valNewPdf( layout%stencil(1)%QQ ) )
  allocate( valMoments( layout%stencil(1)%QQ ) )

  ! Initialize pdf
  write(*,*) "Initialize pdf (oldPDF)"
  do iVal = 1, layout%stencil(1)%QQ
    valOldPdf( iVal ) = real(iVal,rk) * 0.1_rk
  enddo

  ! Convert to moments
  write(*,*) "Convert to moments"
  valMoments = matmul( moment%toMoments%A, valOldPDF )

  ! Check zero and first moments
  write(*,*) "Calculating rho, momX, momY, momZ and their errors"
  rho = sum( valOldPDF )
  mx  = sum( valOldPDF(:) * real(layout%stencil(1)%cxDir(1,:),rk) )
  my  = sum( valOldPDF(:) * real(layout%stencil(1)%cxDir(2,:),rk) )
  mz  = sum( valOldPDF(:) * real(layout%stencil(1)%cxDir(3,:),rk) )
  if ( abs( rho - valMoments(1)) > tolerance ) then
    error = .true.
    write(*,*) " Discrepency in density exceeds tolerance!"
    write(*,*) "   Discrepency = ", abs( rho - valMoments(1))
    write(*,*) "   tolerance   = ", tolerance
  else if ( abs( mx - valMoments(2) ) > tolerance ) then
    error = .true.
    write(*,*) " Discrepency in momX exceeds tolerance!"
    write(*,*) "   Discrepency = ", abs( mx - valMoments(2))
    write(*,*) "   tolerance   = ", tolerance
  else if ( abs( my - valMoments(3) ) > tolerance ) then
    error = .true.
    write(*,*) " Discrepency in momY exceeds tolerance!"
    write(*,*) "   Discrepency = ", abs( mx - valMoments(3))
    write(*,*) "   tolerance   = ", tolerance
  else if ( abs( mz - valMoments(4) ) > tolerance ) then
    error = .true.
    write(*,*) " Discrepency in momY exceeds tolerance!"
    write(*,*) "   Discrepency = ", abs( mx - valMoments(4))
    write(*,*) "   tolerance   = ", tolerance
  end if
  ! todo: high order moments can also be checked

  ! convert back to pdf
  write(*,*) "Convert moments back to pdf (newPDF)"
  valNewPDF = matmul( moment%toPdf%A, valMoments)

  ! check originalPdf and newPdf
  write(*,*) "Calculate difference between newPDF and oldPDF"
  max_error = maxval( abs(valNewPDF - valOldPDF) )
  if ( max_error > tolerance ) then
    error = .true.
    do iVal = 1, layout%stencil(1)%QQ
      write(*, "(A5, 2A20)") 'iDir', 'oldPDF', 'newPDF'
      write(*, "(I5, 2ES20.10)") iVal, valOldPDF(iVal), valNewPDF(iVal)
    end do
  end if

  call tem_finalize(general)
  if (.not. error) then
    write(*,'(A)') 'PASSED'
  else
    write(*,'(A)') 'FAILED'
  end if

end program mus_moments_matrix_test
!******************************************************************************
